# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.aead_key_manager."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import aes_ctr_hmac_aead_pb2
from tink.proto import aes_eax_pb2
from tink.proto import aes_gcm_pb2
from tink.proto import aes_gcm_siv_pb2
from tink.proto import common_pb2
from tink.proto import tink_pb2
from tink.proto import x_aes_gcm_pb2
from tink.proto import xchacha20_poly1305_pb2
import tink
from tink import aead
from tink import core
from tink import mac
from tink.testing import fake_kms


FAKE_KMS_URI = 'fake-kms://CM2b3_MDElQKSAowdHlwZS5nb29nbGVhcGlzLmNvbS9nb29nbGUuY3J5cHRvLnRpbmsuQWVzR2NtS2V5EhIaEIK75t5L-adlUwVhWvRuWUwYARABGM2b3_MDIAE'


def setUpModule():
  aead.register()
  fake_kms.register_client()


class AeadKeyManagerTest(parameterized.TestCase):

  def test_new_key_data_aes_eax(self):
    key_template = aead.aead_key_templates.create_aes_eax_key_template(
        key_size=16, iv_size=12)
    key_manager = core.Registry.key_manager(key_template.type_url)
    key_data = key_manager.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_eax_pb2.AesEaxKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.params.iv_size, 12)
    self.assertLen(key.key_value, 16)

  def test_new_key_data_aes_gcm(self):
    key_template = aead.aead_key_templates.create_aes_gcm_key_template(
        key_size=16)
    key_manager = core.Registry.key_manager(key_template.type_url)
    key_data = key_manager.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_gcm_pb2.AesGcmKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertLen(key.key_value, 16)

  def test_new_key_data_aes_ctr_hmac_aead(self):
    template = aead.aead_key_templates.create_aes_ctr_hmac_aead_key_template(
        aes_key_size=16,
        iv_size=12,
        hmac_key_size=32,
        tag_size=16,
        hash_type=common_pb2.SHA256)
    key_manager = core.Registry.key_manager(template.type_url)
    key_data = key_manager.new_key_data(template)
    self.assertEqual(key_data.type_url, template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_ctr_hmac_aead_pb2.AesCtrHmacAeadKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.aes_ctr_key.version, 0)
    self.assertLen(key.aes_ctr_key.key_value, 16)
    self.assertEqual(key.aes_ctr_key.params.iv_size, 12)
    self.assertEqual(key.hmac_key.version, 0)
    self.assertLen(key.hmac_key.key_value, 32)
    self.assertEqual(key.hmac_key.params.tag_size, 16)
    self.assertEqual(key.hmac_key.params.hash, common_pb2.SHA256)

  def test_new_key_data_aes_gcm_siv(self):
    key_template = aead.aead_key_templates.create_aes_gcm_siv_key_template(
        key_size=16)
    key_manager = core.Registry.key_manager(key_template.type_url)
    key_data = key_manager.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_gcm_siv_pb2.AesGcmSivKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertLen(key.key_value, 16)

  def test_new_key_data_xchacha20_poly1305(self):
    template = aead.aead_key_templates.XCHACHA20_POLY1305
    key_manager = core.Registry.key_manager(template.type_url)
    key_data = key_manager.new_key_data(template)
    self.assertEqual(key_data.type_url, template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = xchacha20_poly1305_pb2.XChaCha20Poly1305Key.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertLen(key.key_value, 32)

  def test_new_key_data_x_aes_gcm(self):
    template = aead.aead_key_templates.XAES_256_GCM_192_BIT_NONCE
    key_manager = core.Registry.key_manager(template.type_url)
    key_data = key_manager.new_key_data(template)
    self.assertEqual(key_data.type_url, template.type_url)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = x_aes_gcm_pb2.XAesGcmKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.params.salt_size, 12)

  def test_invalid_params_throw_exception_aes_eax(self):
    template = aead.aead_key_templates.create_aes_eax_key_template(
        key_size=16, iv_size=9)
    with self.assertRaises(tink.TinkError):
      tink.new_keyset_handle(template)

  def test_invalid_params_throw_exception_aes_gcm(self):
    template = aead.aead_key_templates.create_aes_gcm_key_template(
        key_size=17)
    with self.assertRaises(tink.TinkError):
      tink.new_keyset_handle(template)

  def test_invalid_params_throw_exception_aes_ctr_hmac_aead(self):
    template = aead.aead_key_templates.create_aes_ctr_hmac_aead_key_template(
        aes_key_size=42,
        iv_size=16,
        hmac_key_size=32,
        tag_size=32,
        hash_type=common_pb2.SHA256)
    with self.assertRaises(tink.TinkError):
      tink.new_keyset_handle(template)

  @parameterized.parameters([
      aead.aead_key_templates.AES128_EAX,
      aead.aead_key_templates.AES256_EAX,
      aead.aead_key_templates.AES128_GCM,
      aead.aead_key_templates.AES256_GCM,
      aead.aead_key_templates.AES128_GCM_SIV,
      aead.aead_key_templates.AES256_GCM_SIV,
      aead.aead_key_templates.AES128_CTR_HMAC_SHA256,
      aead.aead_key_templates.AES256_CTR_HMAC_SHA256,
      aead.aead_key_templates.XCHACHA20_POLY1305,
      aead.aead_key_templates.XAES_256_GCM_192_BIT_NONCE,
      aead.aead_key_templates.XAES_256_GCM_192_BIT_NONCE_NO_PREFIX,
      aead.aead_key_templates.XAES_256_GCM_160_BIT_NONCE_NO_PREFIX,
  ])
  def test_encrypt_decrypt_success(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = primitive.encrypt(plaintext, associated_data)
    self.assertEqual(primitive.decrypt(ciphertext, associated_data), plaintext)

  def test_kms_aead_encrypt_decrypt_success(self):
    template = aead.aead_key_templates.create_kms_aead_key_template(
        key_uri=FAKE_KMS_URI)
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = primitive.encrypt(plaintext, associated_data)
    self.assertEqual(primitive.decrypt(ciphertext, associated_data), plaintext)

  def test_kms_aead_with_unknown_key_uri_fails(self):
    template = aead.aead_key_templates.create_kms_aead_key_template(
        key_uri='unknown-kms://key_uri')
    handle = tink.new_keyset_handle(template)
    with self.assertRaises(tink.TinkError):
      handle.primitive(aead.Aead)

  def test_kms_envelope_aead_encrypt_decrypt_success(self):
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri=FAKE_KMS_URI, dek_template=aead.aead_key_templates.AES128_GCM
    )
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = primitive.encrypt(plaintext, associated_data)
    self.assertEqual(primitive.decrypt(ciphertext, associated_data), plaintext)

  def test_kms_envelope_aead_with_unknown_key_uri_fails(self):
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri='unknown-kms://key_uri',
        dek_template=aead.aead_key_templates.AES128_GCM,
    )
    handle = tink.new_keyset_handle(template)
    with self.assertRaises(tink.TinkError):
      handle.primitive(aead.Aead)

  def test_kms_envelope_aead_with_invalid_dek_template_fails(self):
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri=FAKE_KMS_URI,
        dek_template=mac.mac_key_templates.HMAC_SHA256_128BITTAG,
    )
    with self.assertRaises(tink.TinkError):
      _ = tink.new_keyset_handle(template)

  def test_kms_envelope_aead_with_envelope_template_as_dek_template_fails(self):
    env_template = (
        aead.aead_key_templates.create_kms_envelope_aead_key_template(
            kek_uri=FAKE_KMS_URI,
            dek_template=aead.aead_key_templates.AES128_GCM,
        )
    )
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri=FAKE_KMS_URI,
        dek_template=env_template,
    )
    with self.assertRaises(tink.TinkError):
      _ = tink.new_keyset_handle(template)

  def test_kms_envelope_aead_with_kms_template_as_dek_template_fails(self):
    kms_template = aead.aead_key_templates.create_kms_aead_key_template(
        key_uri=FAKE_KMS_URI,
    )
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri=FAKE_KMS_URI,
        dek_template=kms_template,
    )
    with self.assertRaises(tink.TinkError):
      _ = tink.new_keyset_handle(template)

  def test_kms_envelope_aead_decrypt_fixed_ciphertext_success(self):
    # This keyset contains a single KmsEnvelopeAeadKey with
    # kek_uri = FAKE_KMS_URI and dek_template = AES128_GCM.
    json_keyset = '''{
      "primaryKeyId": 371374440,
      "key": [
        {
          "keyData": {
            "typeUrl": "type.googleapis.com/google.crypto.tink.KmsEnvelopeAeadKey",
            "value": "EsMBEjgYARICEBAKMHR5cGUuZ29vZ2xlYXBpcy5jb20vZ29vZ2xlLmNyeXB0by50aW5rLkFlc0djbUtleQqGAWZha2Uta21zOi8vQ00yYjNfTURFbFFLU0Fvd2RIbHdaUzVuYjI5bmJHVmhjR2x6TG1OdmJTOW5iMjluYkdVdVkzSjVjSFJ2TG5ScGJtc3VRV1Z6UjJOdFMyVjVFaElhRUlLNzV0NUwtYWRsVXdWaFd2UnVXVXdZQVJBQkdNMmIzX01ESUFF",
            "keyMaterialType": "REMOTE"
          },
          "status": "ENABLED",
          "keyId": 371374440,
          "outputPrefixType": "RAW"
        }
      ]
    }'''
    keyset_handle = tink.json_proto_keyset_format.parse_without_secret(
        json_keyset
    )

    ciphertext = bytes.fromhex(
        '00000033013e77cdcd39016d632a3bb83b694565d3d2d85329ccc3aff540'
        'ef00a7dcb95c157f0199fa7af825a2da2cf2ed0818e6eb0becb39d73d1b1'
        '26e60bdac2dbe7d742dabaedcc2b3809e429510d9a330dc9e2bb1dc9d5b59ecc'
    )
    primitive = keyset_handle.primitive(aead.Aead)
    decrypted = primitive.decrypt(ciphertext, b'associated_data')
    self.assertEqual(decrypted, b'plaintext')

  @parameterized.parameters([
      aead.aead_key_templates.AES128_EAX,
      aead.aead_key_templates.AES256_EAX,
      aead.aead_key_templates.AES128_GCM,
      aead.aead_key_templates.AES256_GCM,
      aead.aead_key_templates.AES128_GCM_SIV,
      aead.aead_key_templates.AES256_GCM_SIV,
      aead.aead_key_templates.AES128_CTR_HMAC_SHA256,
      aead.aead_key_templates.AES256_CTR_HMAC_SHA256,
      aead.aead_key_templates.XCHACHA20_POLY1305,
      aead.aead_key_templates.XAES_256_GCM_192_BIT_NONCE,
  ])
  def test_invalid_decrypt_raises_error(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    with self.assertRaises(tink.TinkError):
      primitive.decrypt(b'invalid ciphertext', b'ad')


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module defines the interface for MACs (Message Authentication Codes)."""

import abc


class Mac(metaclass=abc.ABCMeta):
  """Interface for MACs (Message Authentication Codes).

  This interface should be used for authentication only, and not for other
  purposes (e.g., it should not be used to generate pseudorandom bytes).
  """

  @abc.abstractmethod
  def compute_mac(self, data: bytes) -> bytes:
    """Computes the message authentication code (MAC) for data.

    Args:
      data: bytes, the input data.
    Returns:
      The resulting MAC as bytes.
    Raises:
      google3.third_party.tink.python.tink.tink_error.TinkError if the
      computation fails.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def verify_mac(self, mac_value: bytes, data: bytes) -> None:
    """Verifies if mac is a correct authentication code (MAC) for data.

    Args:
      mac_value: bytes. The mac to be checked.
      data: bytes. The data to be checked.
    Raises:
      google3.third_party.tink.python.tink.tink_error.TinkError if the
      verification fails.
    """
    raise NotImplementedError()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Constants and convenience methods for the outputs handled by Tink."""

import struct

from tink.proto import tink_pb2
from tink.core import _tink_error

TINK_START_BYTE = b'\x01'
LEGACY_START_BYTE = b'\x00'
RAW_PREFIX_SIZE = 0
NON_RAW_PREFIX_SIZE = 5
TINK_PREFIX_SIZE = NON_RAW_PREFIX_SIZE
RAW_PREFIX = b''


def output_prefix(key: tink_pb2.Keyset.Key) -> bytes:
  """Generates the prefix for the outputs handled by the specified key."""
  if key.output_prefix_type == tink_pb2.TINK:
    return struct.pack('>cL', TINK_START_BYTE, key.key_id)
  elif (key.output_prefix_type == tink_pb2.CRUNCHY or
        key.output_prefix_type == tink_pb2.LEGACY):
    return struct.pack('>cL', LEGACY_START_BYTE, key.key_id)
  elif key.output_prefix_type == tink_pb2.RAW:
    return b''
  else:
    raise _tink_error.TinkError(
        'The given key has invalid OutputPrefixType {}.'.format(
            key.output_prefix_type))



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.testing.keyset_builder."""

from absl.testing import absltest

from tink.proto import tink_pb2
import tink
from tink import aead
from tink import hybrid
from tink.testing import keyset_builder


def setUpModule():
  aead.register()
  hybrid.register()


class KeysetBuilderTest(absltest.TestCase):

  def test_legacy_template(self):
    template = aead.aead_key_templates.AES128_GCM
    legacy_template = keyset_builder.legacy_template(template)
    self.assertEqual(legacy_template.output_prefix_type, tink_pb2.LEGACY)
    self.assertEqual(legacy_template.type_url, template.type_url)
    self.assertEqual(legacy_template.value, template.value)
    # check that generating legacy_template did not change template.
    self.assertNotEqual(template.output_prefix_type, tink_pb2.LEGACY)

  def test_raw_template(self):
    template = aead.aead_key_templates.AES128_GCM
    raw_template = keyset_builder.raw_template(template)
    self.assertEqual(raw_template.output_prefix_type, tink_pb2.RAW)
    self.assertEqual(raw_template.type_url, template.type_url)
    self.assertEqual(raw_template.value, template.value)
   # check that generating raw_template did not change template.
    self.assertNotEqual(template.output_prefix_type, tink_pb2.RAW)

  def test_keyset_handle_conversion(self):
    keyset_handle1 = tink.new_keyset_handle(aead.aead_key_templates.AES128_GCM)
    p1 = keyset_handle1.primitive(aead.Aead)
    builder = keyset_builder.from_keyset_handle(keyset_handle1)
    keyset_handle2 = builder.keyset_handle()
    p2 = keyset_handle2.primitive(aead.Aead)
    ciphertext = p1.encrypt(b'plaintext', b'ad')
    self.assertEqual(p2.decrypt(ciphertext, b'ad'), b'plaintext')

  def test_keyset_conversion(self):
    builder1 = keyset_builder.new_keyset_builder()
    new_key_id = builder1.add_new_key(aead.aead_key_templates.AES128_GCM)
    builder1.set_primary_key(new_key_id)
    keyset = builder1.keyset()
    keyset_handle1 = builder1.keyset_handle()
    p1 = keyset_handle1.primitive(aead.Aead)
    builder2 = keyset_builder.from_keyset(keyset)
    keyset_handle2 = builder2.keyset_handle()
    p2 = keyset_handle2.primitive(aead.Aead)
    ciphertext = p1.encrypt(b'plaintext', b'ad')
    self.assertEqual(p2.decrypt(ciphertext, b'ad'), b'plaintext')

  def test_asymmetric_keyset_conversion(self):
    builder = keyset_builder.new_keyset_builder()
    new_key_id = builder.add_new_key(
        hybrid.hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_GCM)
    builder.set_primary_key(new_key_id)
    private_keyset = builder.keyset()
    public_keyset = builder.public_keyset()
    private_handle = keyset_builder.from_keyset(private_keyset).keyset_handle()
    dec = private_handle.primitive(hybrid.HybridDecrypt)
    public_handle = keyset_builder.from_keyset(public_keyset).keyset_handle()
    enc = public_handle.primitive(hybrid.HybridEncrypt)
    ciphertext = enc.encrypt(b'plaintext', b'context')
    self.assertEqual(dec.decrypt(ciphertext, b'context'), b'plaintext')

  def test_add_new_key_new_id(self):
    builder = keyset_builder.new_keyset_builder()
    key_id1 = builder.add_new_key(aead.aead_key_templates.AES128_GCM)
    key_id2 = builder.add_new_key(aead.aead_key_templates.AES128_GCM)
    self.assertNotEqual(key_id1, key_id2)

  def test_set_primary_success(self):
    builder = keyset_builder.new_keyset_builder()
    secondary_key_id = builder.add_new_key(aead.aead_key_templates.AES128_GCM)
    builder.set_primary_key(secondary_key_id)

  def test_operation_on_unknown_key_fails(self):
    builder = keyset_builder.new_keyset_builder()
    key_id = builder.add_new_key(
        aead.aead_key_templates.AES128_GCM)
    unknown_key_id = key_id + 1
    with self.assertRaises(tink.TinkError):
      builder.set_primary_key(unknown_key_id)
    with self.assertRaises(tink.TinkError):
      builder.enable_key(unknown_key_id)
    with self.assertRaises(tink.TinkError):
      builder.disable_key(unknown_key_id)
    with self.assertRaises(tink.TinkError):
      builder.delete_key(unknown_key_id)

  def test_key_rotation(self):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(aead.aead_key_templates.AES128_GCM)
    builder.set_primary_key(older_key_id)
    p1 = builder.keyset_handle().primitive(aead.Aead)

    newer_key_id = builder.add_new_key(aead.aead_key_templates.AES128_GCM)
    p2 = builder.keyset_handle().primitive(aead.Aead)

    builder.set_primary_key(newer_key_id)
    p3 = builder.keyset_handle().primitive(aead.Aead)

    builder.disable_key(older_key_id)
    p4 = builder.keyset_handle().primitive(aead.Aead)

    self.assertNotEqual(older_key_id, newer_key_id)
    # p1 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    ciphertext1 = p1.encrypt(b'plaintext', b'ad')
    self.assertEqual(p1.decrypt(ciphertext1, b'ad'), b'plaintext')
    self.assertEqual(p2.decrypt(ciphertext1, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext1, b'ad'), b'plaintext')
    with self.assertRaises(tink.TinkError):
      _ = p4.decrypt(ciphertext1, b'ad')

    # p2 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    ciphertext2 = p2.encrypt(b'plaintext', b'ad')
    self.assertEqual(p1.decrypt(ciphertext2, b'ad'), b'plaintext')
    self.assertEqual(p2.decrypt(ciphertext2, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext2, b'ad'), b'plaintext')
    with self.assertRaises(tink.TinkError):
      _ = p4.decrypt(ciphertext2, b'ad')

    # p3 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    ciphertext3 = p3.encrypt(b'plaintext', b'ad')
    with self.assertRaises(tink.TinkError):
      _ = p1.decrypt(ciphertext3, b'ad')
    self.assertEqual(p2.decrypt(ciphertext3, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext3, b'ad'), b'plaintext')
    self.assertEqual(p4.decrypt(ciphertext3, b'ad'), b'plaintext')

    # p4 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    ciphertext4 = p4.encrypt(b'plaintext', b'ad')
    with self.assertRaises(tink.TinkError):
      _ = p1.decrypt(ciphertext4, b'ad')
    self.assertEqual(p2.decrypt(ciphertext4, b'ad'), b'plaintext')
    self.assertEqual(p3.decrypt(ciphertext4, b'ad'), b'plaintext')
    self.assertEqual(p4.decrypt(ciphertext4, b'ad'), b'plaintext')


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A client for Fake KMS."""

import base64
from typing import Optional

import tink
from tink import aead
from tink import secret_key_access


FAKE_KMS_PREFIX = 'fake-kms://'


class FakeKmsClient(tink.KmsClient):
  """A fake KMS client."""

  def __init__(self, key_uri: Optional[str] = None):
    if not key_uri:
      self._key_uri = None
    elif key_uri.startswith(FAKE_KMS_PREFIX):
      self._key_uri = key_uri
    else:
      raise tink.TinkError('invalid key URI')

  def does_support(self, key_uri: str) -> bool:
    if not key_uri.startswith(FAKE_KMS_PREFIX):
      return False
    if not self._key_uri:
      return True
    return key_uri == self._key_uri

  def get_aead(self, key_uri: str) -> aead.Aead:
    if not key_uri.startswith(FAKE_KMS_PREFIX):
      raise tink.TinkError('invalid key URI')
    key_id = key_uri[len(FAKE_KMS_PREFIX) :]
    serialized_key = base64.urlsafe_b64decode(key_id.encode('utf-8') + b'===')
    handle = tink.proto_keyset_format.parse(
        serialized_key, secret_key_access.TOKEN
    )
    return handle.primitive(aead.Aead)


def register_client(key_uri=None, credentials_path=None) -> None:
  """Registers a fake KMS client."""
  _ = credentials_path
  tink.register_kms_client(FakeKmsClient(key_uri))



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._json_util."""

from absl.testing import absltest

from tink.jwt import _json_util
from tink.jwt import _jwt_error


class JwtFormatTest(absltest.TestCase):

  def test_json_dumps(self):
    self.assertEqual(
        _json_util.json_dumps({'a': ['b', 1, True, None]}),
        '{"a":["b",1,true,null]}')

  def test_json_loads(self):
    self.assertEqual(
        _json_util.json_loads('{"a":["b",1,true,null]}'),
        {'a': ['b', 1, True, None]})
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _json_util.json_loads('{invalid')

  def test_json_loads_duplidate_entries_fails(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _json_util.json_loads('{"a":"a1", "a":"a2"}')

  def test_json_loads_with_invalid_utf16(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _json_util.json_loads(u'{"a":{"b":{"c":"\\uD834"}}}')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _json_util.json_loads(u'{"\\uD834":"b"}')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _json_util.json_loads(u'{"a":["a",{"b":["c","\\uD834"]}]}')


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.streaming_aead.decrypting_stream."""

import io
from typing import BinaryIO

from absl.testing import absltest

from tink import core
from tink import streaming_aead
from tink.streaming_aead import _raw_streaming_aead

# Using malformed UTF-8 sequences to ensure there is no accidental decoding.
B_X80 = b'\x80'
B_SOMETHING_ = b'somethin' + B_X80
B_AAD_ = b'aa' + B_X80


def setUpModule():
  streaming_aead.register()


def get_raw_primitive():
  key_data = core.Registry.new_key_data(
      streaming_aead.streaming_aead_key_templates.AES128_CTR_HMAC_SHA256_4KB)
  return core.Registry.primitive(key_data, _raw_streaming_aead.RawStreamingAead)


def get_raw_decrypting_stream(
    ciphertext_source: BinaryIO,
    aad: bytes,
    close_ciphertext_source: bool = True) -> io.RawIOBase:
  return get_raw_primitive().new_raw_decrypting_stream(
      ciphertext_source, aad, close_ciphertext_source=close_ciphertext_source)


class DecryptingStreamTest(absltest.TestCase):

  def test_unsupported_operation(self):
    f = io.BytesIO(B_SOMETHING_)
    ds = get_raw_primitive().new_raw_decrypting_stream(
        f, B_AAD_, close_ciphertext_source=True)

    with self.assertRaises(io.UnsupportedOperation):
      ds.seek(0, 0)
    with self.assertRaises(io.UnsupportedOperation):
      ds.tell()
    with self.assertRaises(io.UnsupportedOperation):
      ds.truncate()
    with self.assertRaises(io.UnsupportedOperation):
      ds.write(b'data')
    with self.assertRaises(io.UnsupportedOperation):
      ds.writelines([b'data'])
    with self.assertRaises(io.UnsupportedOperation):
      ds.fileno()

  def test_closed_methods_raise(self):
    f = io.BytesIO(B_SOMETHING_)
    ds = get_raw_primitive().new_raw_decrypting_stream(
        f, B_AAD_, close_ciphertext_source=True)

    ds.close()
    with self.assertRaisesRegex(ValueError, 'closed'):
      ds.read()
    with self.assertRaisesRegex(ValueError, 'closed'):
      ds.flush()

  def test_inquiries(self):
    f = io.BytesIO(B_SOMETHING_)
    ds = get_raw_primitive().new_raw_decrypting_stream(
        f, B_AAD_, close_ciphertext_source=True)

    self.assertTrue(ds.readable())
    self.assertFalse(ds.writable())
    self.assertFalse(ds.seekable())
    self.assertFalse(ds.isatty())


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.streaming_aead._streaming_aead_wrapper."""

import io
from typing import BinaryIO, cast

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import aes_gcm_hkdf_streaming_pb2
from tink.proto import common_pb2
from tink.proto import tink_pb2
import tink
from tink import secret_key_access
from tink import streaming_aead
from tink.testing import bytes_io
from tink.testing import keyset_builder


TEMPLATE = streaming_aead.streaming_aead_key_templates.AES128_GCM_HKDF_4KB
TYPE_URL = 'type.googleapis.com/google.crypto.tink.AesGcmHkdfStreamingKey'


def setUpModule():
  streaming_aead.register()


def _encrypt(primitive: streaming_aead.StreamingAead, plaintext: bytes,
             associated_data: bytes) -> bytes:
  ciphertext_dest = bytes_io.BytesIOWithValueAfterClose()
  with primitive.new_encrypting_stream(ciphertext_dest, associated_data) as es:
    es.write(plaintext)
  return ciphertext_dest.value_after_close()


class StreamingAeadWrapperTest(parameterized.TestCase):

  @parameterized.parameters(
      [b'plaintext', b'', b'smile \xf0\x9f\x98\x80', b'\xf0\x9f\x98'])
  def test_encrypt_decrypt_success(self, plaintext):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)

    aad = b'associated_data'
    ciphertext_dest = bytes_io.BytesIOWithValueAfterClose()
    with primitive.new_encrypting_stream(ciphertext_dest, aad) as es:
      self.assertLen(plaintext, es.write(plaintext))
    self.assertTrue(ciphertext_dest.closed)

    ciphertext_src = io.BytesIO(ciphertext_dest.value_after_close())
    with primitive.new_decrypting_stream(ciphertext_src, aad) as ds:
      output = ds.read()
    self.assertTrue(ciphertext_src.closed)
    self.assertEqual(output, plaintext)

  def test_long_plaintext_encrypt_decrypt_success(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)

    long_plaintext = b' '.join(b'%d' % i for i in range(10 * 1000 * 1000))
    aad = b'associated_data'
    ciphertext_dest = bytes_io.BytesIOWithValueAfterClose()
    with primitive.new_encrypting_stream(ciphertext_dest, aad) as es:
      self.assertLen(long_plaintext, es.write(long_plaintext))
    self.assertTrue(ciphertext_dest.closed)

    ciphertext_src = io.BytesIO(ciphertext_dest.value_after_close())
    with primitive.new_decrypting_stream(ciphertext_src, aad) as ds:
      output = ds.read()
    self.assertTrue(ciphertext_src.closed)
    self.assertEqual(output, long_plaintext)

  def test_encrypt_decrypt_with_slow_bytes_io_success(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)
    plaintext = b' '.join(b'%d' % i for i in range(10 * 1000))
    aad = b'associated_data'

    # Even if the ciphertext destination only writes small data chunks and
    # sometimes None, calling write() should write the whole plaintext.
    ciphertext_dest = bytes_io.SlowBytesIO()
    with primitive.new_encrypting_stream(ciphertext_dest, aad) as es:
      es.write(plaintext)
    ciphertext = ciphertext_dest.value_after_close()

    # Even if the ciphertext source only returns small data chunks and sometimes
    # None, calling read() should return the whole ciphertext.
    ciphertext_src = cast(BinaryIO, bytes_io.SlowBytesIO(ciphertext))
    with primitive.new_decrypting_stream(ciphertext_src, aad) as ds:
      output = ds.read()
    self.assertTrue(ciphertext_src.closed)
    self.assertEqual(output, plaintext)

  def test_encrypt_decrypt_with_bad_write_bytes_io_success(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)
    plaintext = b'plaintext'
    aad = b'associated_data'

    ciphertext_dest = bytes_io.BadWriteBytesIO()
    with self.assertRaises(tink.TinkError):
      es = primitive.new_encrypting_stream(ciphertext_dest, aad)
      es.write(plaintext)
      es.close()

  def test_encrypt_decrypt_with_slow_readable_raw_bytes_success(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)
    plaintext = b' '.join(b'%d' % i for i in range(10 * 1000))
    aad = b'associated_data'
    ciphertext = _encrypt(primitive, plaintext, aad)

    # Even if the ciphertext source only returns small data chunks and sometimes
    # None, calling read() should return the whole ciphertext.
    ciphertext_src = cast(BinaryIO, bytes_io.SlowReadableRawBytes(ciphertext))
    with primitive.new_decrypting_stream(ciphertext_src, aad) as ds:
      output = ds.read()
    self.assertTrue(ciphertext_src.closed)
    self.assertEqual(output, plaintext)

  def test_encrypt_decrypt_bad_aad(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)

    plaintext = b'plaintext'
    aad = b'associated_data'

    ciphertext_dest = bytes_io.BytesIOWithValueAfterClose()
    with primitive.new_encrypting_stream(ciphertext_dest, aad) as es:
      self.assertLen(plaintext, es.write(plaintext))
    self.assertTrue(ciphertext_dest.closed)

    ciphertext_src = io.BytesIO(ciphertext_dest.value_after_close())
    with primitive.new_decrypting_stream(ciphertext_src, b'bad aad') as ds:
      with self.assertRaises(tink.TinkError):
        _ = ds.read()

  def test_decrypt_unknown_key_fails(self):
    plaintext = b'plaintext'
    aad = b'associated_data'

    unknown_keyset_handle = tink.new_keyset_handle(TEMPLATE)
    unknown_primitive = unknown_keyset_handle.primitive(
        streaming_aead.StreamingAead)
    unknown_ciphertext_dest = bytes_io.BytesIOWithValueAfterClose()
    with unknown_primitive.new_encrypting_stream(unknown_ciphertext_dest,
                                                 aad) as es:
      es.write(plaintext)

    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)
    ciphertext_src = io.BytesIO(unknown_ciphertext_dest.value_after_close())
    with primitive.new_decrypting_stream(ciphertext_src, aad) as ds:
      with self.assertRaises(tink.TinkError):
        _ = ds.read()

  @parameterized.parameters(
      [io.BytesIO, bytes_io.SlowBytesIO, bytes_io.SlowReadableRawBytes])
  def test_encrypt_decrypt_with_key_rotation(self, input_stream_factory):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(TEMPLATE)
    builder.set_primary_key(older_key_id)
    p1 = builder.keyset_handle().primitive(streaming_aead.StreamingAead)

    newer_key_id = builder.add_new_key(TEMPLATE)
    p2 = builder.keyset_handle().primitive(streaming_aead.StreamingAead)

    builder.set_primary_key(newer_key_id)
    p3 = builder.keyset_handle().primitive(streaming_aead.StreamingAead)

    builder.disable_key(older_key_id)
    p4 = builder.keyset_handle().primitive(streaming_aead.StreamingAead)

    self.assertNotEqual(older_key_id, newer_key_id)

    # p1 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    plaintext1 = b' '.join(b'%d' % i for i in range(100 * 101))
    ciphertext1 = _encrypt(p1, plaintext1, b'aad1')
    with p1.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext1)), b'aad1') as ds:
      self.assertEqual(ds.read(), plaintext1)
    with p2.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext1)), b'aad1') as ds:
      self.assertEqual(ds.read(), plaintext1)
    with p3.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext1)), b'aad1') as ds:
      self.assertEqual(ds.read(), plaintext1)
    with p4.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext1)), b'aad1') as ds:
      with self.assertRaises(tink.TinkError):
        ds.read()

    # p2 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    plaintext2 = b' '.join(b'%d' % i for i in range(100 * 102))
    ciphertext2 = _encrypt(p2, plaintext2, b'aad2')
    with p1.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext2)), b'aad2') as ds:
      self.assertEqual(ds.read(), plaintext2)
    with p2.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext2)), b'aad2') as ds:
      self.assertEqual(ds.read(), plaintext2)
    with p3.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext2)), b'aad2') as ds:
      self.assertEqual(ds.read(), plaintext2)
    with p4.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext2)), b'aad2') as ds:
      with self.assertRaises(tink.TinkError):
        ds.read()

    # p3 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    plaintext3 = b' '.join(b'%d' % i for i in range(100 * 103))
    ciphertext3 = _encrypt(p3, plaintext3, b'aad3')
    with p1.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext3)), b'aad3') as ds:
      with self.assertRaises(tink.TinkError):
        ds.read()
    with p2.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext3)), b'aad3') as ds:
      self.assertEqual(ds.read(), plaintext3)
    with p3.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext3)), b'aad3') as ds:
      self.assertEqual(ds.read(), plaintext3)
    with p4.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext3)), b'aad3') as ds:
      self.assertEqual(ds.read(), plaintext3)

    # p4 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    plaintext4 = b' '.join(b'%d' % i for i in range(100 * 104))
    ciphertext4 = _encrypt(p4, plaintext4, b'aad4')
    with p1.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext4)), b'aad4') as ds:
      with self.assertRaises(tink.TinkError):
        ds.read()
    with p2.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext4)), b'aad4') as ds:
      self.assertEqual(ds.read(), plaintext4)
    with p3.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext4)), b'aad4') as ds:
      self.assertEqual(ds.read(), plaintext4)
    with p4.new_decrypting_stream(
        cast(BinaryIO, input_stream_factory(ciphertext4)), b'aad4') as ds:
      self.assertEqual(ds.read(), plaintext4)

  def test_decrypt_tink_output_prefix(self):
    key = aes_gcm_hkdf_streaming_pb2.AesGcmHkdfStreamingKey(
        version=0,
        params=aes_gcm_hkdf_streaming_pb2.AesGcmHkdfStreamingParams(
            ciphertext_segment_size=512,
            derived_key_size=16,
            hkdf_hash_type=common_pb2.HashType.SHA256,
        ),
        key_value=b'0123456789abcdef',
    )
    value1 = key.SerializeToString()
    key.key_value = b'ABCDEF0123456789'
    value2 = key.SerializeToString()

    # We use two keys so that we have at least 1 raw key in the keyset: Tink
    # has a check that creating a new StreamingAead fails when the keyset does
    # not contain any raw keys, and we only want this to fail on decryption.
    keyset = tink_pb2.Keyset(
        primary_key_id=1,
        key=[
            tink_pb2.Keyset.Key(
                key_data=tink_pb2.KeyData(
                    type_url=TYPE_URL,
                    value=value1,
                    key_material_type=tink_pb2.KeyData.SYMMETRIC,
                ),
                output_prefix_type=tink_pb2.OutputPrefixType.TINK,
                status=tink_pb2.KeyStatusType.ENABLED,
                key_id=1,
            ),
            tink_pb2.Keyset.Key(
                key_data=tink_pb2.KeyData(
                    type_url=TYPE_URL,
                    value=value2,
                    key_material_type=tink_pb2.KeyData.SYMMETRIC,
                ),
                output_prefix_type=tink_pb2.OutputPrefixType.RAW,
                status=tink_pb2.KeyStatusType.ENABLED,
                key_id=2,
            ),
        ],
    )

    keyset_handle = tink.proto_keyset_format.parse(
        keyset.SerializeToString(), secret_key_access.TOKEN
    )
    primitive = keyset_handle.primitive(streaming_aead.StreamingAead)

    plaintext = b'plaintext'
    associated_data = b'associated_data'

    ciphertext_dest = bytes_io.BytesIOWithValueAfterClose()
    with primitive.new_encrypting_stream(
        ciphertext_dest, associated_data
    ) as es:
      self.assertLen(plaintext, es.write(plaintext))
    self.assertTrue(ciphertext_dest.closed)

    ciphertext_src = io.BytesIO(ciphertext_dest.value_after_close())
    with primitive.new_decrypting_stream(ciphertext_src, associated_data) as ds:
      output = ds.read()
    self.assertTrue(ciphertext_src.closed)
    self.assertEqual(output, plaintext)


if __name__ == '__main__':
  absltest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/jwt_hmac.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/jwt_hmac.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19tink/proto/jwt_hmac.proto\x12\x12google.crypto.tink\"\xc3\x01\n\nJwtHmacKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x37\n\talgorithm\x18\x02 \x01(\x0e\x32$.google.crypto.tink.JwtHmacAlgorithm\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x12<\n\ncustom_kid\x18\x04 \x01(\x0b\x32(.google.crypto.tink.JwtHmacKey.CustomKid\x1a\x1a\n\tCustomKid\x12\r\n\x05value\x18\x01 \x01(\t\"n\n\x10JwtHmacKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x37\n\talgorithm\x18\x02 \x01(\x0e\x32$.google.crypto.tink.JwtHmacAlgorithm\x12\x10\n\x08key_size\x18\x03 \x01(\r*C\n\x10JwtHmacAlgorithm\x12\x0e\n\nHS_UNKNOWN\x10\x00\x12\t\n\x05HS256\x10\x01\x12\t\n\x05HS384\x10\x02\x12\t\n\x05HS512\x10\x03\x42[\n\x1c\x63om.google.crypto.tink.protoP\x01Z9github.com/tink-crypto/tink-go/v2/proto/jwt_hmac_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.jwt_hmac_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z9github.com/tink-crypto/tink-go/v2/proto/jwt_hmac_go_proto'
  _globals['_JWTHMACALGORITHM']._serialized_start=359
  _globals['_JWTHMACALGORITHM']._serialized_end=426
  _globals['_JWTHMACKEY']._serialized_start=50
  _globals['_JWTHMACKEY']._serialized_end=245
  _globals['_JWTHMACKEY_CUSTOMKID']._serialized_start=219
  _globals['_JWTHMACKEY_CUSTOMKID']._serialized_end=245
  _globals['_JWTHMACKEYFORMAT']._serialized_start=247
  _globals['_JWTHMACKEYFORMAT']._serialized_end=357
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module defines the interface for HybridDecrypt."""

import abc


class HybridDecrypt(metaclass=abc.ABCMeta):
  """The interface for hybrid decryption.

  Implementations of this interface are secure against adaptive
  chosen ciphertext attacks.  In addition to 'plaintext' the
  encryption takes an extra parameter 'context_info', which usually
  is public data implicit from the context, but should be bound to
  the resulting ciphertext: upon decryption the ciphertext allows for
  checking the integrity of 'context_info' (but there are no
  guarantees wrt. to secrecy or authenticity of 'context_info').

  WARNING: hybrid encryption does not provide authenticity, that is the
  recipient of an encrypted message does not know the identity of the sender.
  Similar to general public-key encryption schemes the security goal of
  hybrid encryption is to provide privacy only. In other words, hybrid
  encryption is secure if and only if the recipient can accept anonymous
  messages or can rely on other mechanisms to authenticate the sender.

  'context_info' can be empty or null, but to ensure the correct
  decryption of the resulting ciphertext the same value must be
  provided for decryption operation (cf. HybridDecrypt-interface).

  A concrete instantiation of this interface can implement the
  binding of 'context_info' to the ciphertext in various ways, for
  example:

  - use 'context_info' as "associated data"-input for the employed
    AEAD symmetric encryption (cf. https://tools.ietf.org/html/rfc5116).
  - use 'context_info' as "CtxInfo"-input for HKDF (if the implementation uses
    HKDF as key derivation function, cf. https://tools.ietf.org/html/rfc5869).
  """

  @abc.abstractmethod
  def decrypt(self, ciphertext: bytes, context_info: bytes) -> bytes:
    """Decrypts ciphertext verifying the integrity of context_info."""
    raise NotImplementedError()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.aead_wrapper."""

from absl.testing import absltest
from absl.testing import parameterized

import tink
from tink import _keyset_handle
from tink import _monitoring
from tink import core
from tink import hybrid
from tink import secret_key_access
from tink.testing import fake_key_usage_monitor
from tink.testing import keyset_builder


TEMPLATE = hybrid.hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_GCM
RAW_TEMPLATE = keyset_builder.raw_template(TEMPLATE)

PRIVATE_KEYSET_WITH_PRIMARY = """
  {
    "primaryKeyId": 2160196527,
    "key": [
      {
        "keyData": {
          "typeUrl": "type.googleapis.com/google.crypto.tink.EciesAeadHkdfPrivateKey",
          "value": "GiCP3xXdT0G7GQMQ+k7nsl+W5ElO3EN6ybwSPTGWdArHTBKKASIgrgyjrYI2mV+K8Csa703wGgtlOG+yxHajoM6quNA1IHEaIASUo/ubipc/Vn+WhN64qmnzp9mqzyvQTQ5jrNt6Xwq6EkQYARI6EjgYARICEBAKMHR5cGUuZ29vZ2xlYXBpcy5jb20vZ29vZ2xlLmNyeXB0by50aW5rLkFlc0djbUtleQoEEAMIAg==",
          "keyMaterialType": "ASYMMETRIC_PRIVATE"
        },
        "status": "ENABLED",
        "keyId": 2160196527,
        "outputPrefixType": "TINK"
      }
    ]
  }"""
# Same as PRIVATE_KEYSET_WITH_PRIMARY, but without primaryKeyId.
PRIVATE_KEYSET_WITHOUT_PRIMARY = """
  {
    "key": [
      {
        "keyData": {
          "typeUrl": "type.googleapis.com/google.crypto.tink.EciesAeadHkdfPrivateKey",
          "value": "GiCP3xXdT0G7GQMQ+k7nsl+W5ElO3EN6ybwSPTGWdArHTBKKASIgrgyjrYI2mV+K8Csa703wGgtlOG+yxHajoM6quNA1IHEaIASUo/ubipc/Vn+WhN64qmnzp9mqzyvQTQ5jrNt6Xwq6EkQYARI6EjgYARICEBAKMHR5cGUuZ29vZ2xlYXBpcy5jb20vZ29vZ2xlLmNyeXB0by50aW5rLkFlc0djbUtleQoEEAMIAg==",
          "keyMaterialType": "ASYMMETRIC_PRIVATE"
        },
        "status": "ENABLED",
        "keyId": 2160196527,
        "outputPrefixType": "TINK"
      }
    ]
  }"""
PUBLIC_KEYSET_WITH_PRIMARY = """
  {
    "primaryKeyId": 2160196527,
    "key": [
      {
        "keyData": {
          "typeUrl": "type.googleapis.com/google.crypto.tink.EciesAeadHkdfPublicKey",
          "value": "IiCuDKOtgjaZX4rwKxrvTfAaC2U4b7LEdqOgzqq40DUgcRogBJSj+5uKlz9Wf5aE3riqafOn2arPK9BNDmOs23pfCroSRBgBEjoSOBgBEgIQEAowdHlwZS5nb29nbGVhcGlzLmNvbS9nb29nbGUuY3J5cHRvLnRpbmsuQWVzR2NtS2V5CgQQAwgC",
          "keyMaterialType": "ASYMMETRIC_PUBLIC"
        },
        "status": "ENABLED",
        "keyId": 2160196527,
        "outputPrefixType": "TINK"
      }
    ]
  }"""
# Same as PUBLIC_KEYSET_WITH_PRIMARY, but without primaryKeyId.
PUBLIC_KEYSET_WITHOUT_PRIMARY = """
  {
    "key": [
      {
        "keyData": {
          "typeUrl": "type.googleapis.com/google.crypto.tink.EciesAeadHkdfPublicKey",
          "value": "IiCuDKOtgjaZX4rwKxrvTfAaC2U4b7LEdqOgzqq40DUgcRogBJSj+5uKlz9Wf5aE3riqafOn2arPK9BNDmOs23pfCroSRBgBEjoSOBgBEgIQEAowdHlwZS5nb29nbGVhcGlzLmNvbS9nb29nbGUuY3J5cHRvLnRpbmsuQWVzR2NtS2V5CgQQAwgC",
          "keyMaterialType": "ASYMMETRIC_PUBLIC"
        },
        "status": "ENABLED",
        "keyId": 2160196527,
        "outputPrefixType": "TINK"
      }
    ]
  }"""


def setUpModule():
  hybrid.register()


class HybridWrapperTest(parameterized.TestCase):

  @parameterized.parameters([TEMPLATE, RAW_TEMPLATE])
  def test_encrypt_decrypt(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()

    hybrid_enc = public_handle.primitive(hybrid.HybridEncrypt)
    ciphertext = hybrid_enc.encrypt(b'plaintext', b'context')
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)
    self.assertEqual(hybrid_dec.decrypt(ciphertext, b'context'), b'plaintext')

  @parameterized.parameters([TEMPLATE, RAW_TEMPLATE])
  def test_decrypt_unknown_ciphertext_fails(self, template):
    unknown_private_handle = tink.new_keyset_handle(template)
    unknown_public_handle = unknown_private_handle.public_keyset_handle()
    unknown_enc = unknown_public_handle.primitive(hybrid.HybridEncrypt)
    unknown_ciphertext = unknown_enc.encrypt(b'plaintext', b'context')

    private_handle = tink.new_keyset_handle(template)
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)
    with self.assertRaises(core.TinkError):
      hybrid_dec.decrypt(unknown_ciphertext, b'context')

  @parameterized.parameters([TEMPLATE, RAW_TEMPLATE])
  def test_decrypt_wrong_associated_data_fails(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()

    hybrid_enc = public_handle.primitive(hybrid.HybridEncrypt)
    ciphertext = hybrid_enc.encrypt(b'plaintext', b'context')
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)
    with self.assertRaises(core.TinkError):
      hybrid_dec.decrypt(ciphertext, b'wrong_context')

  @parameterized.parameters([
      (TEMPLATE, TEMPLATE),
      (RAW_TEMPLATE, TEMPLATE),
      (TEMPLATE, RAW_TEMPLATE),
      (RAW_TEMPLATE, RAW_TEMPLATE),
  ])
  def test_encrypt_decrypt_with_key_rotation(self, old_template, new_template):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(old_template)
    builder.set_primary_key(older_key_id)
    private_handle1 = builder.keyset_handle()
    dec1 = private_handle1.primitive(hybrid.HybridDecrypt)
    enc1 = private_handle1.public_keyset_handle().primitive(
        hybrid.HybridEncrypt
    )

    newer_key_id = builder.add_new_key(new_template)
    private_handle2 = builder.keyset_handle()
    dec2 = private_handle2.primitive(hybrid.HybridDecrypt)
    enc2 = private_handle2.public_keyset_handle().primitive(
        hybrid.HybridEncrypt
    )

    builder.set_primary_key(newer_key_id)
    private_handle3 = builder.keyset_handle()
    dec3 = private_handle3.primitive(hybrid.HybridDecrypt)
    enc3 = private_handle3.public_keyset_handle().primitive(
        hybrid.HybridEncrypt
    )

    builder.disable_key(older_key_id)
    private_handle4 = builder.keyset_handle()
    dec4 = private_handle4.primitive(hybrid.HybridDecrypt)
    enc4 = private_handle4.public_keyset_handle().primitive(
        hybrid.HybridEncrypt
    )
    self.assertNotEqual(older_key_id, newer_key_id)

    # p1 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    ciphertext1 = enc1.encrypt(b'plaintext', b'context')
    self.assertEqual(dec1.decrypt(ciphertext1, b'context'), b'plaintext')
    self.assertEqual(dec2.decrypt(ciphertext1, b'context'), b'plaintext')
    self.assertEqual(dec3.decrypt(ciphertext1, b'context'), b'plaintext')
    with self.assertRaises(tink.TinkError):
      _ = dec4.decrypt(ciphertext1, b'context')

    # p2 encrypts with the older key. So p1, p2 and p3 can decrypt it,
    # but not p4.
    ciphertext2 = enc2.encrypt(b'plaintext', b'context')
    self.assertEqual(dec1.decrypt(ciphertext2, b'context'), b'plaintext')
    self.assertEqual(dec2.decrypt(ciphertext2, b'context'), b'plaintext')
    self.assertEqual(dec3.decrypt(ciphertext2, b'context'), b'plaintext')
    with self.assertRaises(tink.TinkError):
      _ = dec4.decrypt(ciphertext2, b'context')

    # p3 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    ciphertext3 = enc3.encrypt(b'plaintext', b'context')
    with self.assertRaises(tink.TinkError):
      _ = dec1.decrypt(ciphertext3, b'context')
    self.assertEqual(dec2.decrypt(ciphertext3, b'context'), b'plaintext')
    self.assertEqual(dec3.decrypt(ciphertext3, b'context'), b'plaintext')
    self.assertEqual(dec4.decrypt(ciphertext3, b'context'), b'plaintext')

    # p4 encrypts with the newer key. So p2, p3 and p4 can decrypt it,
    # but not p1.
    ciphertext4 = enc4.encrypt(b'plaintext', b'context')
    with self.assertRaises(tink.TinkError):
      _ = dec1.decrypt(ciphertext4, b'context')
    self.assertEqual(dec2.decrypt(ciphertext4, b'context'), b'plaintext')
    self.assertEqual(dec3.decrypt(ciphertext4, b'context'), b'plaintext')
    self.assertEqual(dec4.decrypt(ciphertext4, b'context'), b'plaintext')

  def test_encrypt_decrypt_with_primary_key_succeeds(self):
    private_handle_with_primary = tink.json_proto_keyset_format.parse(
        PRIVATE_KEYSET_WITH_PRIMARY, secret_key_access.TOKEN
    )
    public_handle_with_primary = (
        tink.json_proto_keyset_format.parse_without_secret(
            PUBLIC_KEYSET_WITH_PRIMARY
        )
    )
    dec = private_handle_with_primary.primitive(hybrid.HybridDecrypt)
    enc = public_handle_with_primary.primitive(hybrid.HybridEncrypt)
    ciphertext = enc.encrypt(b'plaintext', b'context')
    self.assertEqual(dec.decrypt(ciphertext, b'context'), b'plaintext')

  def test_encrypt_decrypt_without_primary_key_fails(self):
    with self.assertRaises(tink.TinkError):
      tink.json_proto_keyset_format.parse(
          PRIVATE_KEYSET_WITHOUT_PRIMARY, secret_key_access.TOKEN
      )
    # Currently, the public keyset only fails when 'encrypt' is called.
    # TODO(b/228140127): It would be preferable that it fails earlier.
    public_handle_without_primary = (
        tink.json_proto_keyset_format.parse_without_secret(
            PUBLIC_KEYSET_WITHOUT_PRIMARY
        )
    )
    enc_without_primary = public_handle_without_primary.primitive(
        hybrid.HybridEncrypt
    )
    with self.assertRaises(tink.TinkError):
      enc_without_primary.encrypt(b'plaintext', b'context')


class KeyUsageMonitorTest(absltest.TestCase):

  def setUp(self):
    super().setUp()
    self.encrypt_key_usage_monitor = (
        fake_key_usage_monitor.FakeKeyUsageMonitor()
    )
    self.decrypt_key_usage_monitor = (
        fake_key_usage_monitor.FakeKeyUsageMonitor()
    )
    _monitoring.register_key_usage_monitor_factory(
        lambda context: (
            self.encrypt_key_usage_monitor
            if context.get_api_function() == 'encrypt'
            else self.decrypt_key_usage_monitor
        )
    )

  def test_key_usage_monitor_log(self):
    private_handle = _keyset_handle._new_keyset_handle_with_annotations(
        TEMPLATE, {'test': 'test'}
    )
    public_handle = private_handle.public_keyset_handle()
    hybrid_enc = public_handle.primitive(hybrid.HybridEncrypt)

    ciphertext = hybrid_enc.encrypt(b'plaintext', b'context')
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)
    hybrid_dec.decrypt(ciphertext, b'context')

    self.assertSequenceEqual(
        self.encrypt_key_usage_monitor.log_calls,
        [(private_handle.keyset_info().key_info[0].key_id, len(b'plaintext'))],
    )
    self.assertSequenceEqual(
        self.decrypt_key_usage_monitor.log_calls,
        [(
            public_handle.keyset_info().key_info[0].key_id,
            len(ciphertext) - core.crypto_format.NON_RAW_PREFIX_SIZE,
        )],
    )

  def test_key_usage_monitor_log_failure(self):
    private_handle = _keyset_handle._new_keyset_handle_with_annotations(
        TEMPLATE, {'test': 'test'}
    )
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)

    try:
      hybrid_dec.decrypt(b'x', b'context')
    except core.TinkError:
      pass

    self.assertEqual(self.decrypt_key_usage_monitor.log_failure_calls_count, 1)


if __name__ == '__main__':
  absltest.main()



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module provides a global list of KMS Clients."""

import abc
from typing import List

from tink.aead import _aead
from tink.core import _tink_error


class KmsClient(metaclass=abc.ABCMeta):

  @abc.abstractmethod
  def does_support(self, key_uri: str) -> bool:
    raise NotImplementedError()

  @abc.abstractmethod
  def get_aead(self, key_uri: str) -> _aead.Aead:
    raise NotImplementedError()


_kms_clients: List[KmsClient] = []


# Adds client to a global list of KmsClients.
#
# This function should only be called on startup and not on every operation.
#
# In many cases, registering a KMS client is not needed. Instead, get the
# KMS AEAD with kms_aead = client.get_aead(key_uri) and then use it to encrypt
# a keyset with KeysetHandle.write, or to create an envelope AEAD using
# aead.KmsEnvelopeAead.
def register_kms_client(client: KmsClient) -> None:
  """Adds a KMS client to a global list.

  This function should only be called on startup and not on every operation.
  Avoid registering the same client more than once.

  Args:
      client: KmsClient to be registered
  """
  _kms_clients.append(client)


def kms_client_from_uri(key_uri: str) -> KmsClient:
  """Returns the first KMS client that supports key_uri."""
  for client in _kms_clients:
    if client.does_support(key_uri):
      return client
  raise _tink_error.TinkError('No KMS client does support: ' + key_uri)


def reset_kms_clients() -> None:
  """Removes all registered clients. Internal and only used for tests."""
  _kms_clients.clear()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwt_signature_wrappers_test."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import jwt_ecdsa_pb2
from tink.proto import jwt_rsa_ssa_pkcs1_pb2
from tink.proto import jwt_rsa_ssa_pss_pb2
from tink.proto import tink_pb2
import tink
from tink import jwt
from tink import secret_key_access
from tink.jwt import _json_util
from tink.jwt import _jwt_format
from tink.testing import keyset_builder


LONG_CUSTOM_KID = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit'


def setUpModule():
  jwt.register_jwt_signature()


def _change_key_id(keyset_handle: tink.KeysetHandle) -> tink.KeysetHandle:
  """Changes the key id of the first key and makes it primary."""
  serialization = tink.proto_keyset_format.serialize(
      keyset_handle, secret_key_access.TOKEN
  )
  keyset = tink_pb2.Keyset.FromString(serialization)
  # XOR the key id with an arbitrary 32-bit string to get a new key id.
  new_key_id = keyset.key[0].key_id ^ 0xdeadbeef
  keyset.key[0].key_id = new_key_id
  keyset.primary_key_id = new_key_id
  return tink.proto_keyset_format.parse(
      keyset.SerializeToString(), secret_key_access.TOKEN
  )


def _change_output_prefix_to_tink(
    keyset_handle: tink.KeysetHandle) -> tink.KeysetHandle:
  """Changes the output prefix type of the first key to TINK."""
  serialization = tink.proto_keyset_format.serialize(
      keyset_handle, secret_key_access.TOKEN
  )
  keyset = tink_pb2.Keyset.FromString(serialization)
  keyset.key[0].output_prefix_type = tink_pb2.TINK
  return tink.proto_keyset_format.parse(
      keyset.SerializeToString(), secret_key_access.TOKEN
  )


def _set_custom_kid(keyset_handle: tink.KeysetHandle,
                    custom_kid: str) -> tink.KeysetHandle:
  """Sets the custom_kid field of the first key."""
  serialization = tink.proto_keyset_format.serialize(
      keyset_handle, secret_key_access.TOKEN
  )
  keyset = tink_pb2.Keyset.FromString(serialization)
  if keyset.key[0].key_data.type_url.endswith('JwtEcdsaPrivateKey'):
    jwt_ecdsa_key = jwt_ecdsa_pb2.JwtEcdsaPrivateKey.FromString(
        keyset.key[0].key_data.value)
    jwt_ecdsa_key.public_key.custom_kid.value = custom_kid
    keyset.key[0].key_data.value = jwt_ecdsa_key.SerializeToString()
  elif keyset.key[0].key_data.type_url.endswith('JwtRsaSsaPkcs1PrivateKey'):
    rsa_pkcs1_key = jwt_rsa_ssa_pkcs1_pb2.JwtRsaSsaPkcs1PrivateKey.FromString(
        keyset.key[0].key_data.value
    )
    rsa_pkcs1_key.public_key.custom_kid.value = custom_kid
    keyset.key[0].key_data.value = rsa_pkcs1_key.SerializeToString()
  elif keyset.key[0].key_data.type_url.endswith('JwtRsaSsaPssPrivateKey'):
    rsa_pss_key = jwt_rsa_ssa_pss_pb2.JwtRsaSsaPssPrivateKey.FromString(
        keyset.key[0].key_data.value
    )
    rsa_pss_key.public_key.custom_kid.value = custom_kid
    keyset.key[0].key_data.value = rsa_pss_key.SerializeToString()
  else:
    raise tink.TinkError('unknown key type')
  return tink.proto_keyset_format.parse(
      keyset.SerializeToString(), secret_key_access.TOKEN
  )


class JwtSignatureWrapperTest(parameterized.TestCase):

  def test_interesting_error(self):
    private_handle = tink.new_keyset_handle(jwt.jwt_es256_template())
    sign = private_handle.primitive(jwt.JwtPublicKeySign)
    verify = private_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)
    raw_jwt = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)
    compact = sign.sign_and_encode(raw_jwt)
    with self.assertRaisesRegex(jwt.JwtInvalidError,
                                'invalid JWT; expected issuer'):
      verify.verify_and_decode(compact, jwt.new_validator(
          expected_issuer='unknown', allow_missing_expiration=True))

  @parameterized.parameters([
      (jwt.raw_jwt_es256_template(), jwt.raw_jwt_es256_template()),
      (jwt.raw_jwt_es256_template(), jwt.jwt_es256_template()),
      (jwt.jwt_es256_template(), jwt.raw_jwt_es256_template()),
      (jwt.jwt_es256_template(), jwt.jwt_es256_template()),
  ])
  def test_key_rotation(self, old_key_tmpl, new_key_tmpl):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(old_key_tmpl)

    builder.set_primary_key(older_key_id)
    handle1 = builder.keyset_handle()
    sign1 = handle1.primitive(jwt.JwtPublicKeySign)
    verify1 = handle1.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

    newer_key_id = builder.add_new_key(new_key_tmpl)
    handle2 = builder.keyset_handle()
    sign2 = handle2.primitive(jwt.JwtPublicKeySign)
    verify2 = handle2.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

    builder.set_primary_key(newer_key_id)
    handle3 = builder.keyset_handle()
    sign3 = handle3.primitive(jwt.JwtPublicKeySign)
    verify3 = handle3.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

    builder.disable_key(older_key_id)
    handle4 = builder.keyset_handle()
    sign4 = handle4.primitive(jwt.JwtPublicKeySign)
    verify4 = handle4.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

    raw_jwt = jwt.new_raw_jwt(issuer='a', without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='a', allow_missing_expiration=True)

    self.assertNotEqual(older_key_id, newer_key_id)
    # 1 uses the older key. So 1, 2 and 3 can verify the signature, but not 4.
    compact1 = sign1.sign_and_encode(raw_jwt)
    self.assertEqual(
        verify1.verify_and_decode(compact1, validator).issuer(), 'a')
    self.assertEqual(
        verify2.verify_and_decode(compact1, validator).issuer(), 'a')
    self.assertEqual(
        verify3.verify_and_decode(compact1, validator).issuer(), 'a')
    with self.assertRaises(tink.TinkError):
      verify4.verify_and_decode(compact1, validator)

    # 2 uses the older key. So 1, 2 and 3 can verify the signature, but not 4.
    compact2 = sign2.sign_and_encode(raw_jwt)
    self.assertEqual(
        verify1.verify_and_decode(compact2, validator).issuer(), 'a')
    self.assertEqual(
        verify2.verify_and_decode(compact2, validator).issuer(), 'a')
    self.assertEqual(
        verify3.verify_and_decode(compact2, validator).issuer(), 'a')
    with self.assertRaises(tink.TinkError):
      verify4.verify_and_decode(compact2, validator)

    # 3 uses the newer key. So 2, 3 and 4 can verify the signature, but not 1.
    compact3 = sign3.sign_and_encode(raw_jwt)
    with self.assertRaises(tink.TinkError):
      verify1.verify_and_decode(compact3, validator)
    self.assertEqual(
        verify2.verify_and_decode(compact3, validator).issuer(), 'a')
    self.assertEqual(
        verify3.verify_and_decode(compact3, validator).issuer(), 'a')
    self.assertEqual(
        verify4.verify_and_decode(compact3, validator).issuer(), 'a')

    # 4 uses the newer key. So 2, 3 and 4 can verify the signature, but not 1.
    compact4 = sign4.sign_and_encode(raw_jwt)
    with self.assertRaises(tink.TinkError):
      verify1.verify_and_decode(compact4, validator)
    self.assertEqual(
        verify2.verify_and_decode(compact4, validator).issuer(), 'a')
    self.assertEqual(
        verify3.verify_and_decode(compact4, validator).issuer(), 'a')
    self.assertEqual(
        verify4.verify_and_decode(compact4, validator).issuer(), 'a')

  def test_only_tink_output_prefix_type_encodes_a_kid_header(self):
    handle = tink.new_keyset_handle(jwt.raw_jwt_es256_template())
    sign = handle.primitive(jwt.JwtPublicKeySign)
    verify = handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

    tink_handle = _change_output_prefix_to_tink(handle)
    tink_sign = tink_handle.primitive(jwt.JwtPublicKeySign)
    tink_verify = tink_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)

    raw_jwt = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)

    token = sign.sign_and_encode(raw_jwt)
    token_with_kid = tink_sign.sign_and_encode(raw_jwt)

    _, header, _, _ = _jwt_format.split_signed_compact(token)
    self.assertNotIn('kid', _json_util.json_loads(header))

    _, header_with_kid, _, _ = _jwt_format.split_signed_compact(token_with_kid)
    self.assertIn('kid', _json_util.json_loads(header_with_kid))

    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)

    verify.verify_and_decode(token, validator)
    tink_verify.verify_and_decode(token_with_kid, validator)

    other_handle = _change_key_id(tink_handle)
    other_verify = other_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)

    verify.verify_and_decode(token_with_kid, validator)
    # For output prefix type TINK, the kid header is required.
    with self.assertRaises(tink.TinkError):
      tink_verify.verify_and_decode(token, validator)
    # This should fail because value of the kid header is wrong.
    with self.assertRaises(tink.TinkError):
      other_verify.verify_and_decode(token_with_kid, validator)

  @parameterized.named_parameters([
      ('JWT_ES256_RAW', jwt.raw_jwt_es256_template()),
      ('JWT_RS256_RAW', jwt.raw_jwt_rs256_2048_f4_template()),
      ('JWT_PS256_RAW', jwt.raw_jwt_ps256_3072_f4_template()),
  ])
  def test_raw_key_with_custom_kid_header(self, template):
    # normal key with output prefix RAW
    handle = tink.new_keyset_handle(template)
    raw_jwt = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)
    validator = jwt.new_validator(
        expected_issuer='issuer', allow_missing_expiration=True)

    sign = handle.primitive(jwt.JwtPublicKeySign)
    token = sign.sign_and_encode(raw_jwt)
    verify = handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)
    verify.verify_and_decode(token, validator)

    _, json_header, _, _ = _jwt_format.split_signed_compact(token)
    self.assertNotIn('kid', _json_util.json_loads(json_header))

    # key with a custom_kid set
    custom_kid_handle = _set_custom_kid(handle, custom_kid=LONG_CUSTOM_KID)
    custom_kid_sign = custom_kid_handle.primitive(jwt.JwtPublicKeySign)
    token_with_kid = custom_kid_sign.sign_and_encode(raw_jwt)
    custom_kid_verify = custom_kid_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)
    custom_kid_verify.verify_and_decode(token_with_kid, validator)

    _, header_with_kid, _, _ = _jwt_format.split_signed_compact(token_with_kid)
    self.assertEqual(_json_util.json_loads(header_with_kid)['kid'],
                     LONG_CUSTOM_KID)

    # The primitive with a custom_kid set accepts tokens without kid header.
    custom_kid_verify.verify_and_decode(token, validator)

    # The primitive without a custom_kid set ignores the kid header.
    verify.verify_and_decode(token_with_kid, validator)

    # key with a different custom_kid set
    other_handle = _set_custom_kid(handle, custom_kid='other kid')
    other_verify = other_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)
    # Fails because the kid value do not match.
    with self.assertRaises(tink.TinkError):
      other_verify.verify_and_decode(token_with_kid, validator)

    tink_handle = _change_output_prefix_to_tink(custom_kid_handle)
    tink_sign = tink_handle.primitive(jwt.JwtPublicKeySign)
    tink_verify = tink_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)
    # Having custom_kid set with output prefix TINK is not allowed.
    with self.assertRaises(tink.TinkError):
      tink_sign.sign_and_encode(raw_jwt)
    with self.assertRaises(tink.TinkError):
      tink_verify.verify_and_decode(token, validator)
    with self.assertRaises(tink.TinkError):
      tink_verify.verify_and_decode(token_with_kid, validator)

  def test_legacy_template_fails(self):
    template = keyset_builder.legacy_template(jwt.jwt_es256_template())
    builder = keyset_builder.new_keyset_builder()
    key_id = builder.add_new_key(template)
    builder.set_primary_key(key_id)
    handle = builder.keyset_handle()
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeySign)
    with self.assertRaises(tink.TinkError):
      handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

  def test_legacy_non_primary_key_fails(self):
    builder = keyset_builder.new_keyset_builder()
    old_template = keyset_builder.legacy_template(jwt.jwt_es256_template())
    _ = builder.add_new_key(old_template)
    current_key_id = builder.add_new_key(jwt.jwt_es256_template())
    builder.set_primary_key(current_key_id)
    handle = builder.keyset_handle()
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeySign)
    with self.assertRaises(tink.TinkError):
      handle.public_keyset_handle().primitive(jwt.JwtPublicKeyVerify)

  def test_jwt_mac_from_keyset_without_primary_fails(self):
    builder = keyset_builder.new_keyset_builder()
    builder.add_new_key(jwt.jwt_es256_template())
    with self.assertRaises(tink.TinkError):
      builder.keyset_handle()


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Pre-generated KeyTemplate for Aead.

One can use these templates to generate a new tink_pb2.Keyset with
tink_pb2.KeysetHandle. To generate a new keyset that contains a single
aes_eax_pb2.AesEaxKey, one can do:
handle = keyset_handle.KeysetHandle(aead_key_templates.AES128_EAX).
"""

from tink.proto import aes_cmac_prf_pb2
from tink.proto import common_pb2
from tink.proto import hkdf_prf_pb2
from tink.proto import hmac_prf_pb2
from tink.proto import tink_pb2

_AES_CMAC_PRF_KEY_TYPE_URL = (
    'type.googleapis.com/google.crypto.tink.AesCmacPrfKey')
_HMAC_PRF_KEY_TYPE_URL = 'type.googleapis.com/google.crypto.tink.HmacPrfKey'
_HKDF_PRF_KEY_TYPE_URL = 'type.googleapis.com/google.crypto.tink.HkdfPrfKey'


def _create_aes_cmac_key_template(key_size: int) -> tink_pb2.KeyTemplate:
  """Creates an AES CMAC PRF KeyTemplate, and fills in its values."""
  key_format = aes_cmac_prf_pb2.AesCmacPrfKeyFormat()
  key_format.key_size = key_size
  key_format.version = 0
  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_AES_CMAC_PRF_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.RAW,
  )
  return key_template


def _create_hmac_key_template(
    key_size: int, hash_type: common_pb2.HashType) -> tink_pb2.KeyTemplate:
  """Creates an HMAC PRF KeyTemplate, and fills in its values."""
  key_format = hmac_prf_pb2.HmacPrfKeyFormat()
  key_format.params.hash = hash_type
  key_format.key_size = key_size
  key_format.version = 0
  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_HMAC_PRF_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.RAW,
  )
  return key_template


def _create_hkdf_key_template(
    key_size: int, hash_type: common_pb2.HashType) -> tink_pb2.KeyTemplate:
  """Creates an HKDF PRF KeyTemplate, and fills in its values."""
  key_format = hkdf_prf_pb2.HkdfPrfKeyFormat()
  key_format.params.hash = hash_type
  key_format.key_size = key_size
  key_format.version = 0
  key_template = tink_pb2.KeyTemplate()
  key_template.value = key_format.SerializeToString()
  key_template.type_url = _HKDF_PRF_KEY_TYPE_URL
  key_template.output_prefix_type = tink_pb2.RAW
  return key_template


AES_CMAC = _create_aes_cmac_key_template(key_size=32)
HMAC_SHA256 = _create_hmac_key_template(
    key_size=32, hash_type=common_pb2.SHA256)
HMAC_SHA512 = _create_hmac_key_template(
    key_size=64, hash_type=common_pb2.SHA512)
HKDF_SHA256 = _create_hkdf_key_template(
    key_size=32, hash_type=common_pb2.SHA256)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.deterministic_aead_key_manager."""

from absl.testing import absltest

from tink.proto import aes_siv_pb2
from tink.proto import tink_pb2
import tink
from tink import core
from tink import daead


def setUpModule():
  daead.register()


class DeterministicAeadKeyManagerTest(absltest.TestCase):

  def test_new_key_data(self):
    key_template = daead.deterministic_aead_key_templates.AES256_SIV
    key_manager = core.Registry.key_manager(key_template.type_url)
    key_data = key_manager.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_manager.key_type())
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.SYMMETRIC)
    key = aes_siv_pb2.AesSivKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertLen(key.key_value, 64)

  def test_invalid_params_throw_exception(self):
    key_template = (daead.deterministic_aead_key_templates
                    .create_aes_siv_key_template(63))
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  def test_encrypt_decrypt(self):
    keyset_handle = tink.new_keyset_handle(
        daead.deterministic_aead_key_templates.AES256_SIV)
    daead_primitive = keyset_handle.primitive(daead.DeterministicAead)
    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = daead_primitive.encrypt_deterministically(
        plaintext, associated_data)
    self.assertEqual(
        daead_primitive.decrypt_deterministically(ciphertext, associated_data),
        plaintext)

  def test_invalid_decrypt_raises_error(self):
    keyset_handle = tink.new_keyset_handle(
        daead.deterministic_aead_key_templates.AES256_SIV)
    daead_primitive = keyset_handle.primitive(daead.DeterministicAead)
    with self.assertRaises(core.TinkError):
      daead_primitive.decrypt_deterministically(
          b'bad ciphertext', b'associated_data')


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module defines the basic types and abstract classes in Tink."""

import abc
from typing import Any, Generic, Type, TypeVar

from tink.proto import tink_pb2
from tink.core import _tink_error


P = TypeVar('P')


class KeyManager(Generic[P], metaclass=abc.ABCMeta):
  """Generates keys and provides primitives for the keys.

  A KeyManager "understands" keys of a specific key types: it can generate keys
  of a supported type and create primitives for supported keys.  A key type is
  identified by the global name of the protocol buffer that holds the
  corresponding key material, and is given by type_url-field of KeyData-protocol
  buffer.
  """

  @abc.abstractmethod
  def primitive_class(self) -> Type[P]:
    """The class of the primitive it uses. Used for internal management."""
    raise NotImplementedError()

  @abc.abstractmethod
  def primitive(self, key_data: tink_pb2.KeyData) -> P:
    """Constructs an primitive for the given key.

    Args:
      key_data: KeyData protocol buffer
    Returns:
      A primitive, for example an instance of Aead or Mac.
    Raises:
      tink.TinkError if getting the primitive fails.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def key_type(self) -> str:
    """Returns the type_url identifying the key type handled by this manager."""
    raise NotImplementedError()

  @abc.abstractmethod
  def new_key_data(self,
                   key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    """Generates a new random key, based on the specified key_template.

    Args:
      key_template: KeyTemplate protocol buffer
    Returns:
      A KeyData protocol buffer that contains the key.
    Raises:
      tink.TinkError if the key generation fails.
    """
    raise NotImplementedError()

  def does_support(self, type_url: str) -> bool:
    return self.key_type() == type_url


class PrivateKeyManager(KeyManager[P]):
  """Generates keys and provides primitives for the keys."""

  @abc.abstractmethod
  def public_key_data(
      self, private_key_data: tink_pb2.KeyData) -> tink_pb2.KeyData:
    """Generates a new random key, based on the specified key_template.

    Args:
      private_key_data: KeyData protocol buffer
    Returns:
      A KeyData protocol buffer that contains the public key.
    Raises:
      tink.TinkError if the key generation fails.
    """
    raise NotImplementedError()


class KeyManagerCcToPyWrapper(KeyManager[P]):
  """Transforms C++ KeyManager into a Python KeyManager."""

  def __init__(self,
               cc_key_manager: Any,  # A pybinded CcKeyManager<P> instance
               primitive_class: Type[P],
               primitive_py_wrapper: Type[P]):
    self._cc_key_manager = cc_key_manager
    self._primitive_class = primitive_class
    self._primitive_py_wrapper = primitive_py_wrapper

  def primitive_class(self) -> Type[P]:
    return self._primitive_class

  @_tink_error.use_tink_errors
  def primitive(self, key_data: tink_pb2.KeyData) -> P:
    return self._primitive_py_wrapper(
        self._cc_key_manager.primitive(key_data.SerializeToString()))

  def key_type(self) -> str:
    return self._cc_key_manager.key_type()

  @_tink_error.use_tink_errors
  def new_key_data(self,
                   key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    return tink_pb2.KeyData.FromString(
        self._cc_key_manager.new_key_data(key_template.SerializeToString()))


class PrivateKeyManagerCcToPyWrapper(PrivateKeyManager[P]):
  """Transforms C++ KeyManager into a Python KeyManager."""

  def __init__(self,
               cc_key_manager: Any,  # A pybinded CcKeyManager<P> instance
               primitive_class: Type[P],
               primitive_py_wrapper: Type[P]):
    self._cc_key_manager = cc_key_manager
    self._primitive_class = primitive_class
    self._primitive_py_wrapper = primitive_py_wrapper

  def primitive_class(self) -> Type[P]:
    return self._primitive_class

  @_tink_error.use_tink_errors
  def primitive(self, key_data: tink_pb2.KeyData) -> P:
    return self._primitive_py_wrapper(
        self._cc_key_manager.primitive(key_data.SerializeToString()))

  def key_type(self) -> str:
    return self._cc_key_manager.key_type()

  @_tink_error.use_tink_errors
  def new_key_data(self,
                   key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    return tink_pb2.KeyData.FromString(
        self._cc_key_manager.new_key_data(key_template.SerializeToString()))

  @_tink_error.use_tink_errors
  def public_key_data(self, key_data: tink_pb2.KeyData) -> tink_pb2.KeyData:
    return tink_pb2.KeyData.FromString(
        self._cc_key_manager.public_key_data(key_data.SerializeToString()))



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.testing.bytes_io."""

import io

from absl.testing import absltest

from tink.testing import bytes_io


class BytesIoTest(absltest.TestCase):

  def test_close_success(self):
    f = bytes_io.BytesIOWithValueAfterClose()
    f.write(b'foobar')
    f.close()
    self.assertEqual(f.value_after_close(), b'foobar')

  def test_with_text_wrapper_success(self):
    f = bytes_io.BytesIOWithValueAfterClose()
    with io.TextIOWrapper(f, encoding='utf8') as t:
      t.write(u'foobar')
    self.assertTrue(f.closed)
    value = f.value_after_close()
    self.assertEqual(value, u'foobar'.encode('utf8'))

  def test_initial_bytes_success(self):
    f = bytes_io.BytesIOWithValueAfterClose(b'abc')
    f.write(b'foobar')
    f.close()
    self.assertEqual(f.value_after_close(), b'foobar')

  def test_value_before_close_fails(self):
    f = bytes_io.BytesIOWithValueAfterClose()
    f.write(b'foobar')
    with self.assertRaises(ValueError):
      f.value_after_close()
    f.close()

  def test_close_twice_success(self):
    f = bytes_io.BytesIOWithValueAfterClose(b'abc')
    f.write(b'foobar')
    f.close()
    f.close()
    self.assertEqual(f.value_after_close(), b'foobar')


class SlowBytesIOTest(absltest.TestCase):

  def test_read(self):
    with bytes_io.SlowBytesIO(b'The quick brown fox') as f:
      with self.assertRaises(io.BlockingIOError):
        f.read(10)
      self.assertEqual(b'The q', f.read(10))
      self.assertEqual(b'uick ', f.read(10))
      with self.assertRaises(io.BlockingIOError):
        f.read(10)
      self.assertEqual(b'brown', f.read(10))
      self.assertEqual(b' fox', f.read(10))
      with self.assertRaises(io.BlockingIOError):
        f.read(10)
      self.assertEqual(b'', f.read(10))
    self.assertTrue(f.closed)

  def test_read_no_argument(self):
    with bytes_io.SlowBytesIO(b'The quick brown fox') as f:
      self.assertEqual(b'The quick brown fox', f.read())
      self.assertEqual(b'', f.read())

  def test_read_minus_one(self):
    with bytes_io.SlowBytesIO(b'The quick brown fox') as f:
      self.assertEqual(b'The quick brown fox', f.read(-1))
      self.assertEqual(b'', f.read(-1))

  def test_not_seekable(self):
    with bytes_io.SlowBytesIO(b'The quick brown fox', seekable=False) as f:
      self.assertFalse(f.seekable())
      with self.assertRaises(io.UnsupportedOperation):
        f.seek(0)

  def test_seekable(self):
    with bytes_io.SlowBytesIO(b'The quick brown fox', seekable=True) as f:
      self.assertTrue(f.seekable())
      self.assertEqual(b'The quick brown fox', f.read())
      f.seek(0)
      self.assertEqual(b'The quick brown fox', f.read())

  def test_write(self):
    with bytes_io.SlowBytesIO() as f:
      with self.assertRaises(io.BlockingIOError) as e:
        f.write(b'The quick ')
      self.assertEqual(e.exception.characters_written, 3)
      len1 = f.write(b'brown fox ')
      self.assertEqual(len1, 5)
      len2 = f.write(b'jumps over ')
      self.assertEqual(len2, 5)
      with self.assertRaises(io.BlockingIOError) as e2:
        f.write(b'')
      self.assertEqual(e2.exception.characters_written, 0)
      len3 = f.write(b'dog')
      self.assertEqual(len3, 3)
      len4 = f.write(b'')
      self.assertEqual(len4, 0)
    self.assertEqual(b'Thebrownjumpsdog', f.value_after_close())


class SlowReadableRawBytesTest(absltest.TestCase):

  def test_read(self):
    with bytes_io.SlowReadableRawBytes(b'The quick brown fox') as f:
      self.assertIsNone(f.read(10))
      self.assertEqual(b'The q', f.read(10))
      self.assertEqual(b'uick ', f.read(10))
      self.assertIsNone(f.read(10))
      self.assertEqual(b'brown', f.read(10))
      self.assertEqual(b' fox', f.read(10))
      self.assertIsNone(f.read(10))
      self.assertEqual(b'', f.read(10))
    self.assertTrue(f.closed)

  # Note that the documentation of readall(), read() and read(-1) is wrong:
  # https://docs.python.org/2/library/io.html#io.RawIOBase.readall.
  # readall does multiple reads, but it stops as soon as it can't read any data.
  # The documentation says that it reads until it reaches EOF.
  def test_readall(self):
    with bytes_io.SlowReadableRawBytes(b'The quick brown fox') as f:
      self.assertIsNone(None, f.readall())
      self.assertEqual(b'The quick ', f.readall())
      self.assertEqual(b'brown fox', f.readall())
      self.assertEqual(b'', f.readall())

  def test_read_no_argument(self):
    with bytes_io.SlowReadableRawBytes(b'The quick brown fox') as f:
      self.assertIsNone(None, f.read())
      self.assertEqual(b'The quick ', f.read())
      self.assertEqual(b'brown fox', f.read())
      self.assertEqual(b'', f.read())

  def test_read_minus_one(self):
    with bytes_io.SlowReadableRawBytes(b'The quick brown fox') as f:
      self.assertIsNone(None, f.read(-1))
      self.assertEqual(b'The quick ', f.read(-1))
      self.assertEqual(b'brown fox', f.read(-1))
      self.assertEqual(b'', f.read(-1))

  def test_not_seekable(self):
    with bytes_io.SlowReadableRawBytes(
        b'The quick brown fox', seekable=False) as f:
      self.assertFalse(f.seekable())
      with self.assertRaises(io.UnsupportedOperation):
        f.seek(0)

  def test_seekable(self):
    with bytes_io.SlowReadableRawBytes(
        b'The quick brown fox', seekable=True) as f:
      self.assertTrue(f.seekable())
      self.assertIsNone(None, f.read())
      self.assertEqual(b'The quick ', f.read())
      f.seek(0)
      self.assertEqual(b'The quick ', f.read())


class BadWriteBytesIOTest(absltest.TestCase):

  def test_write(self):
    with bytes_io.BadWriteBytesIO() as f:
      len1 = f.write(b'aaa')
      self.assertEqual(len1, 4)  # 3 + 1
      len2 = f.write(b'')
      self.assertEqual(len2, 1)  # 0 + 1


if __name__ == '__main__':
  absltest.main()



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for kms_clients."""

from absl.testing import absltest
import tink
from tink import _kms_clients
from tink import aead
from tink import tink_config


def setUpModule():
  tink_config.register()


class FakeClient(_kms_clients.KmsClient):

  def __init__(self, key_uri):
    self.key_uri = key_uri

  def does_support(self, key_uri: str) -> bool:
    return key_uri == self.key_uri

  def get_aead(self, key_uri: str) -> aead.Aead:
    raise ValueError('unknown key_uri')


class KmsClientsTest(absltest.TestCase):

  def test_register_get_and_reset_kms_clients(self):
    client1 = FakeClient('key_uri1')
    client2 = FakeClient('key_uri2')
    client3 = FakeClient('key_uri3')
    tink.register_kms_client(client1)
    tink.register_kms_client(client2)
    tink.register_kms_client(client3)

    # returns the first registered client that supports the uri.
    self.assertEqual(tink.kms_client_from_uri('key_uri1'), client1)
    self.assertEqual(tink.kms_client_from_uri('key_uri2'), client2)
    with self.assertRaises(tink.TinkError):
      tink.kms_client_from_uri('unknown')

    _kms_clients.reset_kms_clients()
    with self.assertRaises(tink.TinkError):
      tink.kms_client_from_uri('key_uri1')


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.deterministic_aead_key_templates."""

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import aes_siv_pb2
from tink.proto import tink_pb2
from tink import daead


class DeterministicAeadKeyTemplatesTest(parameterized.TestCase):

  def test_create_aes_siv_key_template(self):
    # Intentionally using 'weird' or invalid values for parameters,
    # to test that the function correctly puts them in the resulting template.
    template = None
    with self.assertWarns(DeprecationWarning):
      template = (
          daead.deterministic_aead_key_templates.create_aes_siv_key_template(
              key_size=42))
    self.assertEqual('type.googleapis.com/google.crypto.tink.AesSivKey',
                     template.type_url)
    self.assertEqual(tink_pb2.TINK, template.output_prefix_type)
    key_format = aes_siv_pb2.AesSivKeyFormat()
    key_format.ParseFromString(template.value)
    self.assertEqual(42, key_format.key_size)

if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.crypto_format."""

from absl.testing import absltest
from tink.proto import tink_pb2
from tink import core


def to_byte(c):
  # items in byte strings are of type str in Python v2.7 and int in v3.5
  if isinstance(c, str):
    return c.encode()
  else:
    return chr(c).encode()


class CryptoFormatTest(absltest.TestCase):

  def test_tink_prefix(self):
    key = tink_pb2.Keyset.Key()
    key.output_prefix_type = tink_pb2.TINK
    key.key_id = 0x00040695
    prefix = core.crypto_format.output_prefix(key)
    self.assertLen(prefix, core.crypto_format.TINK_PREFIX_SIZE)
    self.assertEqual(
        to_byte(prefix[0]), bytes(core.crypto_format.TINK_START_BYTE))
    # key_id in big-endian format.
    self.assertEqual(prefix[1:5], b'\x00\x04\x06\x95')

  def test_legacy_prefix(self):
    key = tink_pb2.Keyset.Key()
    key.output_prefix_type = tink_pb2.LEGACY
    key.key_id = 0xFF7F1058
    prefix = core.crypto_format.output_prefix(key)
    self.assertLen(prefix, core.crypto_format.NON_RAW_PREFIX_SIZE)
    self.assertEqual(to_byte(prefix[0]), core.crypto_format.LEGACY_START_BYTE)
    # key_id in big-endian format.
    self.assertEqual(prefix[1:5], b'\xFF\x7F\x10\x58')

  def test_crunchy_prefix(self):
    key = tink_pb2.Keyset.Key()
    key.output_prefix_type = tink_pb2.CRUNCHY
    key.key_id = 0x12AAB1
    prefix = core.crypto_format.output_prefix(key)
    self.assertLen(prefix, core.crypto_format.NON_RAW_PREFIX_SIZE)
    self.assertEqual(to_byte(prefix[0]), core.crypto_format.LEGACY_START_BYTE)
    # key_id in big-endian format.
    self.assertEqual(prefix[1:5], b'\x00\x12\xAA\xB1')

  def test_raw_prefix(self):
    key = tink_pb2.Keyset.Key()
    key.output_prefix_type = tink_pb2.RAW
    key.key_id = 0x74EB33
    prefix = core.crypto_format.output_prefix(key)
    self.assertLen(prefix, core.crypto_format.RAW_PREFIX_SIZE)

  def test_invalid_output_prefix(self):
    key = tink_pb2.Keyset.Key()
    key.output_prefix_type = 42
    key.key_id = 0x11223344
    with self.assertRaises(core.TinkError):
      _ = core.crypto_format.output_prefix(key)


if __name__ == '__main__':
  absltest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/prf_based_deriver.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/prf_based_deriver.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import tink_pb2 as tink_dot_proto_dot_tink__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"tink/proto/prf_based_deriver.proto\x12\x12google.crypto.tink\x1a\x15tink/proto/tink.proto\"V\n\x15PrfBasedDeriverParams\x12=\n\x14\x64\x65rived_key_template\x18\x01 \x01(\x0b\x32\x1f.google.crypto.tink.KeyTemplate\"\x90\x01\n\x18PrfBasedDeriverKeyFormat\x12\x39\n\x10prf_key_template\x18\x01 \x01(\x0b\x32\x1f.google.crypto.tink.KeyTemplate\x12\x39\n\x06params\x18\x02 \x01(\x0b\x32).google.crypto.tink.PrfBasedDeriverParams\"\x8e\x01\n\x12PrfBasedDeriverKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12,\n\x07prf_key\x18\x02 \x01(\x0b\x32\x1b.google.crypto.tink.KeyData\x12\x39\n\x06params\x18\x03 \x01(\x0b\x32).google.crypto.tink.PrfBasedDeriverParamsBd\n\x1c\x63om.google.crypto.tink.protoP\x01ZBgithub.com/tink-crypto/tink-go/v2/proto/prf_based_deriver_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.prf_based_deriver_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZBgithub.com/tink-crypto/tink-go/v2/proto/prf_based_deriver_go_proto'
  _globals['_PRFBASEDDERIVERPARAMS']._serialized_start=81
  _globals['_PRFBASEDDERIVERPARAMS']._serialized_end=167
  _globals['_PRFBASEDDERIVERKEYFORMAT']._serialized_start=170
  _globals['_PRFBASEDDERIVERKEYFORMAT']._serialized_end=314
  _globals['_PRFBASEDDERIVERKEY']._serialized_start=317
  _globals['_PRFBASEDDERIVERKEY']._serialized_end=459
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Static methods for handling of Tink configurations."""

from tink import aead
from tink import daead
from tink import hybrid
from tink import mac
from tink import prf
from tink import signature
from tink import streaming_aead


def register():
  aead.register()
  daead.register()
  hybrid.register()
  mac.register()
  prf.register()
  signature.register()
  streaming_aead.register()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.integration.gcp_kms_client."""

import os

from absl.testing import absltest
from absl.testing import parameterized
from google.api_core import exceptions as core_exceptions
from google.cloud import kms_v1

from tink import core
from tink.integration import gcpkms
from tink.testing import helper


KEY_URI1 = 'gcp-kms://projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1'
KEY_URI2 = 'gcp-kms://projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck2'
KEY_URI3 = 'gcp-kms://projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck3/cryptoKeyVersions/1'
AWS_KEY_URI = 'aws-kms://arn:aws:kms:us-west-2:acc:other/key3'
PLAINTEXT = b'plaintext'
CIPHERTEXT = b'ciphertext'
ASSOCIATED_DATA = b'associated_data'
CREDENTIAL_PATH = os.path.join(
    helper.tink_py_testdata_path(), 'gcp/credential.json'
)


class CustomException(core_exceptions.GoogleAPIError):
  pass


class GcpKmsClientTest(parameterized.TestCase):

  def setUp(self):
    super().setUp()
    absltest.mock.patch.object(kms_v1, 'KeyManagementServiceClient').start()

  def tearDown(self):
    absltest.mock.patch.stopall()
    super().tearDown()

  @parameterized.parameters(
      (KEY_URI1, True),
      (KEY_URI2, False),
      (KEY_URI1 + 'suffix', False),
      (AWS_KEY_URI, False),
  )
  def test_client_bound_to_key_uri(self, key_uri, expected_support):
    gcp_client = gcpkms.GcpKmsClient(KEY_URI1, CREDENTIAL_PATH)
    self.assertEqual(gcp_client.does_support(key_uri), expected_support)

  @parameterized.parameters(
      (KEY_URI1, True),
      (KEY_URI2, True),
      (KEY_URI1 + 'suffix', True),
      (AWS_KEY_URI, False),
  )
  def test_client_not_bound_to_key_uri(self, key_uri, expected_support):
    gcp_client = gcpkms.GcpKmsClient(None, CREDENTIAL_PATH)
    self.assertEqual(gcp_client.does_support(key_uri), expected_support)

  @parameterized.parameters(
      (KEY_URI1, True),
      (KEY_URI2, True),
      (AWS_KEY_URI, False),
  )
  def test_client_empty_key_uri(self, key_uri, expected_support):
    gcp_client = gcpkms.GcpKmsClient('', CREDENTIAL_PATH)
    self.assertEqual(gcp_client.does_support(key_uri), expected_support)

  def test_client_invalid_path(self):
    with self.assertRaises(FileNotFoundError):
      gcpkms.GcpKmsClient(None, CREDENTIAL_PATH + 'corrupted')

  @parameterized.parameters(
      '',
      AWS_KEY_URI,
      KEY_URI1 + '/',
      'projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1',
      'gcp-kms:/projects/p1/locations/global/keyRings/kr1/cryptoKeys/ck1',
  )
  def test_aead_wrong_key_uri_fails(self, key_uri):
    gcp_client = gcpkms.GcpKmsClient(None, CREDENTIAL_PATH)
    with self.assertRaises(core.TinkError):
      gcp_client.get_aead(key_uri)

  def test_aead_different_key_uri_fails(self):
    gcp_client = gcpkms.GcpKmsClient(KEY_URI1, CREDENTIAL_PATH)
    with self.assertRaises(core.TinkError):
      gcp_client.get_aead(KEY_URI1 + 'suffix')

  def test_aead_encryption_fails(self):
    kms_v1.KeyManagementServiceClient().encrypt.side_effect = CustomException()
    gcp_client = gcpkms.GcpKmsClient(KEY_URI1, CREDENTIAL_PATH)
    gcp_aead = gcp_client.get_aead(KEY_URI1)
    with self.assertRaises(core.TinkError):
      gcp_aead.encrypt(CIPHERTEXT, ASSOCIATED_DATA)

  def test_aead_decryption_fails(self):
    kms_v1.KeyManagementServiceClient().decrypt.side_effect = CustomException()
    gcp_client = gcpkms.GcpKmsClient(KEY_URI1, CREDENTIAL_PATH)
    gcp_aead = gcp_client.get_aead(KEY_URI1)
    with self.assertRaises(core.TinkError):
      gcp_aead.decrypt(PLAINTEXT, ASSOCIATED_DATA)

  @parameterized.parameters(
      KEY_URI1,
      KEY_URI2,
      KEY_URI3,
  )
  def test_aead_encryption_works(self, key_uri):
    kms_v1.KeyManagementServiceClient().encrypt.return_value = (
        kms_v1.types.EncryptResponse(ciphertext=CIPHERTEXT)
    )
    gcp_client = gcpkms.GcpKmsClient(key_uri, CREDENTIAL_PATH)
    gcp_aead = gcp_client.get_aead(key_uri)
    ciphertext = gcp_aead.encrypt(PLAINTEXT, ASSOCIATED_DATA)
    self.assertEqual(ciphertext, CIPHERTEXT)

  @parameterized.parameters(
      KEY_URI1,
      KEY_URI2,
  )
  def test_aead_decryption_works(self, key_uri):
    kms_v1.KeyManagementServiceClient().decrypt.return_value = (
        kms_v1.types.DecryptResponse(plaintext=PLAINTEXT)
    )
    gcp_client = gcpkms.GcpKmsClient(key_uri, CREDENTIAL_PATH)
    gcp_aead = gcp_client.get_aead(key_uri)
    plaintext = gcp_aead.decrypt(CIPHERTEXT, ASSOCIATED_DATA)
    self.assertEqual(plaintext, PLAINTEXT)

  def test_aead_decryption_with_key_version_fails(self):
    gcp_client = gcpkms.GcpKmsClient(KEY_URI3, CREDENTIAL_PATH)
    gcp_aead = gcp_client.get_aead(KEY_URI3)
    with self.assertRaises(core.TinkError):
      gcp_aead.decrypt(CIPHERTEXT, ASSOCIATED_DATA)


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
"""The JwtValidator."""

import datetime

from typing import Optional, cast
from tink.jwt import _jwt_error
from tink.jwt import _raw_jwt

_MAX_CLOCK_SKEW = datetime.timedelta(minutes=10)


class JwtValidator:
  """A JwtValidator defines how JSON Web Tokens (JWTs) should be validated.

    By default, the JwtValidator requires that a token has a valid expiration
    claim, no issuer and no audience claim. This can be changed using the
    expect_... and  ignore_... arguments.

    If present, the JwtValidator also validates the not-before claim. The
    validation time can be changed using the fixed_now parameter. clock_skew can
    be set to allow a small leeway (not more than 10 minutes) to account for
    clock skew.
  """

  def __init__(self,
               *,
               expected_type_header: Optional[str],
               expected_issuer: Optional[str],
               expected_audience: Optional[str],
               ignore_type_header: bool,
               ignore_issuer: bool,
               ignore_audiences: bool,
               allow_missing_expiration: bool,
               expect_issued_in_the_past: bool,
               clock_skew: Optional[datetime.timedelta],
               fixed_now: Optional[datetime.datetime]) -> None:
    if expected_type_header and ignore_type_header:
      raise ValueError(
          'expected_type_header and ignore_type_header cannot be used together')
    if expected_issuer and ignore_issuer:
      raise ValueError(
          'expected_issuer and ignore_issuer cannot be used together')
    if expected_audience and ignore_audiences:
      raise ValueError(
          'expected_audience and ignore_audiences cannot be used together')
    self._expected_type_header = expected_type_header
    self._expected_issuer = expected_issuer
    self._expected_audience = expected_audience
    self._ignore_type_header = ignore_type_header
    self._ignore_issuer = ignore_issuer
    self._ignore_audiences = ignore_audiences
    self._allow_missing_expiration = allow_missing_expiration
    self._expect_issued_in_the_past = expect_issued_in_the_past
    if clock_skew:
      if clock_skew > _MAX_CLOCK_SKEW:
        raise ValueError('clock skew too large, max is 10 minutes')
      self._clock_skew = cast(datetime.timedelta, clock_skew)
    else:
      self._clock_skew = datetime.timedelta()
    if fixed_now and not fixed_now.tzinfo:
      raise ValueError('fixed_now without tzinfo')
    self._fixed_now = fixed_now

  def has_expected_type_header(self) -> bool:
    return self._expected_type_header is not None

  def expected_type_header(self) -> Optional[str]:
    return self._expected_type_header

  def has_expected_issuer(self) -> bool:
    return self._expected_issuer is not None

  def expected_issuer(self) -> Optional[str]:
    return self._expected_issuer

  def has_expected_audience(self) -> bool:
    return self._expected_audience is not None

  def expected_audience(self) -> Optional[str]:
    return self._expected_audience

  def ignore_type_header(self) -> bool:
    return self._ignore_type_header

  def ignore_issuer(self) -> bool:
    return self._ignore_issuer

  def ignore_audiences(self) -> bool:
    return self._ignore_audiences

  def allow_missing_expiration(self) -> bool:
    return self._allow_missing_expiration

  def expect_issued_in_the_past(self) -> bool:
    return self._expect_issued_in_the_past

  def clock_skew(self) -> datetime.timedelta:
    return self._clock_skew

  def has_fixed_now(self) -> bool:
    return self._fixed_now is not None

  def fixed_now(self) -> Optional[datetime.datetime]:
    return self._fixed_now


def new_validator(
    *,
    expected_type_header: Optional[str] = None,
    expected_issuer: Optional[str] = None,
    expected_audience: Optional[str] = None,
    ignore_type_header: bool = False,
    ignore_issuer: bool = False,
    ignore_audiences: bool = False,
    allow_missing_expiration: bool = False,
    expect_issued_in_the_past: bool = False,
    clock_skew: Optional[datetime.timedelta] = None,
    fixed_now: Optional[datetime.datetime] = None) -> JwtValidator:
  """Creates a new JwtValidator."""
  return JwtValidator(
      expected_type_header=expected_type_header,
      expected_issuer=expected_issuer,
      expected_audience=expected_audience,
      ignore_type_header=ignore_type_header,
      ignore_issuer=ignore_issuer,
      ignore_audiences=ignore_audiences,
      allow_missing_expiration=allow_missing_expiration,
      expect_issued_in_the_past=expect_issued_in_the_past,
      clock_skew=clock_skew,
      fixed_now=fixed_now)


def validate(validator: JwtValidator, raw_jwt: _raw_jwt.RawJwt) -> None:
  """Validates a jwt.RawJwt and raises JwtInvalidError if it is invalid.

  This function is called by the JWT primitives and does not need to be called
  by the user.

  Args:
    validator: a jwt.JwtValidator that defines how to validate tokens.
    raw_jwt: a jwt.RawJwt token to validate.
  Raises:
    jwt.JwtInvalidError
  """
  if validator.has_fixed_now():
    now = cast(datetime.datetime, validator.fixed_now())
  else:
    now = datetime.datetime.now(tz=datetime.timezone.utc)
  if not raw_jwt.has_expiration() and not validator.allow_missing_expiration():
    raise _jwt_error.JwtInvalidError('token is missing an expiration')
  if (raw_jwt.has_expiration() and
      raw_jwt.expiration() <= now - validator.clock_skew()):
    raise _jwt_error.JwtInvalidError('token has expired since %s' %
                                     raw_jwt.expiration())
  if (raw_jwt.has_not_before() and
      raw_jwt.not_before() > now + validator.clock_skew()):
    raise _jwt_error.JwtInvalidError('token cannot be used before %s' %
                                     raw_jwt.not_before())
  if validator.expect_issued_in_the_past():
    if not raw_jwt.has_issued_at():
      raise _jwt_error.JwtInvalidError('token is missing iat claim')
    if raw_jwt.issued_at() > now + validator.clock_skew():
      raise _jwt_error.JwtInvalidError(
          'token has a invalid iat claim in the future: %s' %
          raw_jwt.issued_at())
  if validator.has_expected_type_header():
    if not raw_jwt.has_type_header():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; missing expected type header %s.' %
          validator.expected_type_header())
    if validator.expected_type_header() != raw_jwt.type_header():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; expected type header %s, but got %s' %
          (validator.expected_type_header(), raw_jwt.type_header()))
  else:
    if raw_jwt.has_type_header() and not validator.ignore_type_header():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; token has type_header set, but validator not.')
  if validator.has_expected_issuer():
    if not raw_jwt.has_issuer():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; missing expected issuer %s.' %
          validator.expected_issuer())
    if validator.expected_issuer() != raw_jwt.issuer():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; expected issuer %s, but got %s' %
          (validator.expected_issuer(), raw_jwt.issuer()))
  else:
    if raw_jwt.has_issuer() and not validator.ignore_issuer():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; token has issuer set, but validator not.')
  if validator.has_expected_audience():
    if (not raw_jwt.has_audiences() or
        validator.expected_audience() not in raw_jwt.audiences()):
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; missing expected audience %s.' %
          validator.expected_audience())
  else:
    if raw_jwt.has_audiences() and not validator.ignore_audiences():
      raise _jwt_error.JwtInvalidError(
          'invalid JWT; token has audience set, but validator not.')



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Internal-only module that gives access to the proto keyset."""

from tink.proto import tink_pb2
from tink import _keyset_handle
from tink import _secret_key_access
from tink import core


def from_proto_keyset(
    keyset: tink_pb2.Keyset, token: core.KeyAccess
) -> _keyset_handle.KeysetHandle:
  if not isinstance(token, _secret_key_access.SecretKeyAccess):
    raise core.TinkError('no secret access.')
  return _keyset_handle.KeysetHandle._create(keyset)  # pylint: disable=protected-access


def to_proto_keyset(
    keyset_handle: _keyset_handle.KeysetHandle, token: core.KeyAccess
) -> tink_pb2.Keyset:
  if not isinstance(token, _secret_key_access.SecretKeyAccess):
    raise core.TinkError('no secret access.')
  return keyset_handle._keyset  # pylint: disable=protected-access



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PrfSet package."""

from tink.prf import _prf_key_manager
from tink.prf import _prf_key_templates as prf_key_templates
from tink.prf import _prf_set

Prf = _prf_set.Prf
PrfSet = _prf_set.PrfSet
register = _prf_key_manager.register



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A Raw Input stream wrapper that supports rewinding."""

import io
from typing import Optional, BinaryIO


class RewindableInputStream(io.RawIOBase):
  """Implements a readable io.RawIOBase wrapper that supports rewinding.

  The wrapped input_stream can either be a io.RawIOBase or io.BufferedIOBase.
  """

  def __init__(self, input_stream: BinaryIO):
    super().__init__()
    if not input_stream.readable():
      raise ValueError('input_stream must be readable')
    self._input_stream = input_stream
    self._buffer = bytearray()
    self._pos = 0
    self._rewindable = True

  def read(self, size: int = -1) -> Optional[bytes]:
    """Read and return up to size bytes when size >= 0.

    If input_stream.read returns None to indicate "No data at the moment", this
    function may return None as well. But it will eventually return
    some data, or return b'' if EOF is reached.

    Args:
      size: Maximum number of bytes to be returned, if >= 0. If size is smaller
        than 0 or None, return the whole content of the file.
    Returns:
      bytes read. b'' is returned on EOF, and None if there is currently
      no data available, but EOF is not reached yet.
    """
    if size is None or size < 0:
      return self.readall()  # implemented in io.RawIOBase
    if self._pos < len(self._buffer):
      # buffer has some data left. Return up to 'size' bytes from the buffer
      new_pos = min(len(self._buffer), self._pos + size)
      b = self._buffer[self._pos:new_pos]
      self._pos = new_pos
      return bytes(b)
    # no data left in buffer
    if not self._rewindable and self._buffer:
      # buffer is not needed anymore
      self._buffer = bytearray()
      self._pos = 0
    try:
      data = self._input_stream.read(size)
    except BlockingIOError:
      # self._input_stream is a BufferedIOBase and has currently no data
      return None
    if data is None:
      # self._input_stream is a RawIOBase and has currently no data
      return None
    if self._rewindable:
      self._buffer.extend(data)
      self._pos += len(data)
    return data

  def rewind(self) -> None:
    if not self._rewindable:
      raise ValueError('rewind is disabled')
    self._pos = 0

  def disable_rewind(self) -> None:
    self._rewindable = False

  def readable(self) -> bool:
    return True

  def close(self) -> None:
    """Close the stream and the wrapped input_stream."""
    if self.closed:  # pylint:disable=using-constant-test
      return
    self._input_stream.close()
    super().close()
